@@ -1,10 +1,10 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
+ * contributor license agreements. See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
+ * the License. You may obtain a copy of the License at
  *
  *    http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -18,10 +18,14 @@ package org.apache.kafka.common.metrics;
 
 import java.util.List;
 
+import org.apache.kafka.common.Configurable;
+
 /**
- * A plugin interface to allow things to listen as new metrics are created so they can be reported
+ * A plugin interface to allow things to listen as new metrics are created so they can be reported.
+ * <p>
+ * Implement {@link org.apache.kafka.common.ClusterResourceListener} to receive cluster metadata once it's available. Please see the class documentation for ClusterResourceListener for more information.
  */
-public interface MetricsReporter {
+public interface MetricsReporter extends Configurable {
 
     /**
      * This is called when the reporter is first registered to initially register all existing metrics
@@ -35,6 +39,12 @@ public interface MetricsReporter {
      */
     public void metricChange(KafkaMetric metric);
 
+    /**
+     * This is called whenever a metric is removed
+     * @param metric
+     */
+    public void metricRemoval(KafkaMetric metric);
+
     /**
      * Called when the metrics repository is closed.
      */
