@@ -1,10 +1,10 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
+ * contributor license agreements. See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
+ * the License. You may obtain a copy of the License at
  *
  *    http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -18,6 +18,9 @@ package org.apache.kafka.clients;
 
 import org.apache.kafka.common.Cluster;
 import org.apache.kafka.common.Node;
+import org.apache.kafka.common.errors.AuthenticationException;
+import org.apache.kafka.common.errors.InterruptException;
+import org.apache.kafka.common.errors.UnsupportedVersionException;
 import org.apache.kafka.common.requests.AbstractRequest;
 import org.apache.kafka.common.requests.AbstractResponse;
 import org.apache.kafka.common.utils.Time;
@@ -47,17 +50,23 @@ public class MockClient implements KafkaClient {
         }
     };
 
-    private class FutureResponse {
-        public final AbstractResponse responseBody;
-        public final boolean disconnected;
-        public final RequestMatcher requestMatcher;
-        public Node node;
+    private static class FutureResponse {
+        private final Node node;
+        private final RequestMatcher requestMatcher;
+        private final AbstractResponse responseBody;
+        private final boolean disconnected;
+        private final boolean isUnsupportedRequest;
 
-        public FutureResponse(AbstractResponse responseBody, boolean disconnected, RequestMatcher requestMatcher, Node node) {
+        public FutureResponse(Node node,
+                              RequestMatcher requestMatcher,
+                              AbstractResponse responseBody,
+                              boolean disconnected,
+                              boolean isUnsupportedRequest) {
+            this.node = node;
+            this.requestMatcher = requestMatcher;
             this.responseBody = responseBody;
             this.disconnected = disconnected;
-            this.requestMatcher = requestMatcher;
-            this.node = node;
+            this.isUnsupportedRequest = isUnsupportedRequest;
         }
 
     }
@@ -65,7 +74,7 @@ public class MockClient implements KafkaClient {
     private final Time time;
     private final Metadata metadata;
     private Set<String> unavailableTopics;
-    private int correlation = 0;
+    private Cluster cluster;
     private Node node = null;
     private final Set<String> ready = new HashSet<>();
     private final Map<Node, Long> blackedOut = new HashMap<>();
@@ -76,6 +85,7 @@ public class MockClient implements KafkaClient {
     private final Queue<FutureResponse> futureResponses = new ArrayDeque<>();
     private final Queue<MetadataUpdate> metadataUpdates = new ArrayDeque<>();
     private volatile NodeApiVersions nodeApiVersions = NodeApiVersions.create();
+    private volatile int numBlockingWakeups = 0;
 
     public MockClient(Time time) {
         this(time, null);
@@ -127,13 +137,20 @@ public class MockClient implements KafkaClient {
         return isBlackedOut(node);
     }
 
+    @Override
+    public AuthenticationException authenticationException(Node node) {
+        return null;
+    }
+
+    @Override
     public void disconnect(String node) {
         long now = time.milliseconds();
         Iterator<ClientRequest> iter = requests.iterator();
         while (iter.hasNext()) {
             ClientRequest request = iter.next();
             if (request.destination().equals(node)) {
-                responses.add(new ClientResponse(request.makeHeader(), request.callback(), request.destination(),
+                short version = request.requestBuilder().latestAllowedVersion();
+                responses.add(new ClientResponse(request.makeHeader(version), request.callback(), request.destination(),
                         request.createdTimeMs(), now, true, null, null));
                 iter.remove();
             }
@@ -148,14 +165,22 @@ public class MockClient implements KafkaClient {
             FutureResponse futureResp = iterator.next();
             if (futureResp.node != null && !request.destination().equals(futureResp.node.idString()))
                 continue;
-            request.requestBuilder().setVersion(nodeApiVersions.usableVersion(
-                    request.requestBuilder().apiKey()));
-            AbstractRequest abstractRequest = request.requestBuilder().build();
-            if (!futureResp.requestMatcher.matches(abstractRequest))
-                throw new IllegalStateException("Next in line response did not match expected request");
 
-            ClientResponse resp = new ClientResponse(request.makeHeader(), request.callback(), request.destination(),
-                    request.createdTimeMs(), time.milliseconds(), futureResp.disconnected, null, futureResp.responseBody);
+            AbstractRequest.Builder<?> builder = request.requestBuilder();
+            short version = nodeApiVersions.latestUsableVersion(request.apiKey(), builder.oldestAllowedVersion(),
+                    builder.latestAllowedVersion());
+            AbstractRequest abstractRequest = request.requestBuilder().build(version);
+            if (!futureResp.requestMatcher.matches(abstractRequest))
+                throw new IllegalStateException("Request matcher did not match next-in-line request " + abstractRequest);
+
+            UnsupportedVersionException unsupportedVersionException = null;
+            if (futureResp.isUnsupportedRequest)
+                unsupportedVersionException = new UnsupportedVersionException("Api " +
+                        request.apiKey() + " with version " + version);
+
+            ClientResponse resp = new ClientResponse(request.makeHeader(version), request.callback(), request.destination(),
+                    request.createdTimeMs(), time.milliseconds(), futureResp.disconnected,
+                    unsupportedVersionException, futureResp.responseBody);
             responses.add(resp);
             iterator.remove();
             return;
@@ -164,22 +189,62 @@ public class MockClient implements KafkaClient {
         this.requests.add(request);
     }
 
+    /**
+     * Simulate a blocking poll in order to test wakeup behavior.
+     *
+     * @param numBlockingWakeups The number of polls which will block until woken up
+     */
+    public synchronized void enableBlockingUntilWakeup(int numBlockingWakeups) {
+        this.numBlockingWakeups = numBlockingWakeups;
+    }
+
+    @Override
+    public synchronized void wakeup() {
+        if (numBlockingWakeups > 0) {
+            numBlockingWakeups--;
+            notify();
+        }
+    }
+
+    private synchronized void maybeAwaitWakeup() {
+        try {
+            int remainingBlockingWakeups = numBlockingWakeups;
+            if (remainingBlockingWakeups <= 0)
+                return;
+
+            while (numBlockingWakeups == remainingBlockingWakeups)
+                wait();
+        } catch (InterruptedException e) {
+            throw new InterruptException(e);
+        }
+    }
+
     @Override
     public List<ClientResponse> poll(long timeoutMs, long now) {
+        maybeAwaitWakeup();
+
         List<ClientResponse> copy = new ArrayList<>(this.responses);
 
         if (metadata != null && metadata.updateRequested()) {
             MetadataUpdate metadataUpdate = metadataUpdates.poll();
+            if (cluster != null)
+                metadata.update(cluster, this.unavailableTopics, time.milliseconds());
             if (metadataUpdate == null)
                 metadata.update(metadata.fetch(), this.unavailableTopics, time.milliseconds());
             else {
+                if (metadataUpdate.expectMatchRefreshTopics
+                    && !metadata.topics().equals(metadataUpdate.cluster.topics())) {
+                    throw new IllegalStateException("The metadata topics does not match expectation. "
+                                                        + "Expected topics: " + metadataUpdate.cluster.topics()
+                                                        + ", asked topics: " + metadata.topics());
+                }
                 this.unavailableTopics = metadataUpdate.unavailableTopics;
                 metadata.update(metadataUpdate.cluster, metadataUpdate.unavailableTopics, time.milliseconds());
             }
         }
 
-        while (!this.responses.isEmpty()) {
-            ClientResponse response = this.responses.poll();
+        ClientResponse response;
+        while ((response = this.responses.poll()) != null) {
             response.onComplete();
         }
 
@@ -194,9 +259,32 @@ public class MockClient implements KafkaClient {
         respond(response, false);
     }
 
+    public void respond(RequestMatcher matcher, AbstractResponse response) {
+        ClientRequest nextRequest = requests.peek();
+        if (nextRequest == null)
+            throw new IllegalStateException("No current requests queued");
+
+        AbstractRequest request = nextRequest.requestBuilder().build();
+        if (!matcher.matches(request))
+            throw new IllegalStateException("Request matcher did not match next-in-line request " + request);
+
+        respond(response);
+    }
+
+    // Utility method to enable out of order responses
+    public void respondToRequest(ClientRequest clientRequest, AbstractResponse response) {
+        AbstractRequest request = clientRequest.requestBuilder().build();
+        requests.remove(clientRequest);
+        short version = clientRequest.requestBuilder().latestAllowedVersion();
+        responses.add(new ClientResponse(clientRequest.makeHeader(version), clientRequest.callback(), clientRequest.destination(),
+                clientRequest.createdTimeMs(), time.milliseconds(), false, null, response));
+    }
+
+
     public void respond(AbstractResponse response, boolean disconnected) {
         ClientRequest request = requests.remove();
-        responses.add(new ClientResponse(request.makeHeader(), request.callback(), request.destination(),
+        short version = request.requestBuilder().latestAllowedVersion();
+        responses.add(new ClientResponse(request.makeHeader(version), request.callback(), request.destination(),
                 request.createdTimeMs(), time.milliseconds(), disconnected, null, response));
     }
 
@@ -210,7 +298,8 @@ public class MockClient implements KafkaClient {
             ClientRequest request = iterator.next();
             if (request.destination().equals(node.idString())) {
                 iterator.remove();
-                responses.add(new ClientResponse(request.makeHeader(), request.callback(), request.destination(),
+                short version = request.requestBuilder().latestAllowedVersion();
+                responses.add(new ClientResponse(request.makeHeader(version), request.callback(), request.destination(),
                         request.createdTimeMs(), time.milliseconds(), disconnected, null, response));
                 return;
             }
@@ -223,7 +312,7 @@ public class MockClient implements KafkaClient {
     }
 
     public void prepareResponseFrom(AbstractResponse response, Node node) {
-        prepareResponseFrom(ALWAYS_TRUE, response, node, false);
+        prepareResponseFrom(ALWAYS_TRUE, response, node, false, false);
     }
 
     /**
@@ -237,7 +326,7 @@ public class MockClient implements KafkaClient {
     }
 
     public void prepareResponseFrom(RequestMatcher matcher, AbstractResponse response, Node node) {
-        prepareResponseFrom(matcher, response, node, false);
+        prepareResponseFrom(matcher, response, node, false, false);
     }
 
     public void prepareResponse(AbstractResponse response, boolean disconnected) {
@@ -245,22 +334,35 @@ public class MockClient implements KafkaClient {
     }
 
     public void prepareResponseFrom(AbstractResponse response, Node node, boolean disconnected) {
-        prepareResponseFrom(ALWAYS_TRUE, response, node, disconnected);
+        prepareResponseFrom(ALWAYS_TRUE, response, node, disconnected, false);
     }
 
     /**
      * Prepare a response for a request matching the provided matcher. If the matcher does not
-     * match, {@link KafkaClient#send(ClientRequest, long)} will throw IllegalStateException
-     * @param matcher The matcher to apply
+     * match, {@link KafkaClient#send(ClientRequest, long)} will throw IllegalStateException.
+     * @param matcher The request matcher to apply
      * @param response The response body
      * @param disconnected Whether the request was disconnected
      */
     public void prepareResponse(RequestMatcher matcher, AbstractResponse response, boolean disconnected) {
-        prepareResponseFrom(matcher, response, null, disconnected);
+        prepareResponseFrom(matcher, response, null, disconnected, false);
     }
 
-    public void prepareResponseFrom(RequestMatcher matcher, AbstractResponse response, Node node, boolean disconnected) {
-        futureResponses.add(new FutureResponse(response, disconnected, matcher, node));
+    /**
+     * Raise an unsupported version error on the next request if it matches the given matcher.
+     * If the matcher does not match, {@link KafkaClient#send(ClientRequest, long)} will throw IllegalStateException.
+     * @param matcher The request matcher to apply
+     */
+    public void prepareUnsupportedVersionResponse(RequestMatcher matcher) {
+        prepareResponseFrom(matcher, null, null, false, true);
+    }
+
+    private void prepareResponseFrom(RequestMatcher matcher,
+                                     AbstractResponse response,
+                                     Node node,
+                                     boolean disconnected,
+                                     boolean isUnsupportedVersion) {
+        futureResponses.add(new FutureResponse(node, matcher, response, disconnected, isUnsupportedVersion));
     }
 
     public void waitForRequests(final int minRequests, long maxWaitMs) throws InterruptedException {
@@ -281,22 +383,60 @@ public class MockClient implements KafkaClient {
         metadataUpdates.clear();
     }
 
+    public boolean hasPendingMetadataUpdates() {
+        return !metadataUpdates.isEmpty();
+    }
+
     public void prepareMetadataUpdate(Cluster cluster, Set<String> unavailableTopics) {
-        metadataUpdates.add(new MetadataUpdate(cluster, unavailableTopics));
+        metadataUpdates.add(new MetadataUpdate(cluster, unavailableTopics, false));
+    }
+
+    public void prepareMetadataUpdate(Cluster cluster,
+                                      Set<String> unavailableTopics,
+                                      boolean expectMatchMetadataTopics) {
+        metadataUpdates.add(new MetadataUpdate(cluster, unavailableTopics, expectMatchMetadataTopics));
     }
 
     public void setNode(Node node) {
         this.node = node;
     }
 
+    public void cluster(Cluster cluster) {
+        this.cluster = cluster;
+    }
+
     @Override
     public int inFlightRequestCount() {
         return requests.size();
     }
 
+    @Override
+    public boolean hasInFlightRequests() {
+        return !requests.isEmpty();
+    }
+
+    public boolean hasPendingResponses() {
+        return !responses.isEmpty();
+    }
+
     @Override
     public int inFlightRequestCount(String node) {
-        return requests.size();
+        int result = 0;
+        for (ClientRequest req : requests) {
+            if (req.destination().equals(node))
+                ++result;
+        }
+        return result;
+    }
+
+    @Override
+    public boolean hasInFlightRequests(String node) {
+        return inFlightRequestCount(node) > 0;
+    }
+
+    @Override
+    public boolean hasReadyNodes() {
+        return !ready.isEmpty();
     }
 
     @Override
@@ -312,10 +452,6 @@ public class MockClient implements KafkaClient {
                 expectResponse, callback);
     }
 
-    @Override
-    public void wakeup() {
-    }
-
     @Override
     public void close() {
     }
@@ -347,9 +483,11 @@ public class MockClient implements KafkaClient {
     private static class MetadataUpdate {
         final Cluster cluster;
         final Set<String> unavailableTopics;
-        MetadataUpdate(Cluster cluster, Set<String> unavailableTopics) {
+        final boolean expectMatchRefreshTopics;
+        MetadataUpdate(Cluster cluster, Set<String> unavailableTopics, boolean expectMatchRefreshTopics) {
             this.cluster = cluster;
             this.unavailableTopics = unavailableTopics;
+            this.expectMatchRefreshTopics = expectMatchRefreshTopics;
         }
     }
 }
