@@ -1,10 +1,10 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
+ * contributor license agreements. See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
+ * the License. You may obtain a copy of the License at
  *
  *    http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -56,6 +56,7 @@ public class MockConsumerInterceptor implements ClusterResourceListener, Consume
     }
 
     @Override
+    @SuppressWarnings("deprecation")
     public ConsumerRecords<String, String> onConsume(ConsumerRecords<String, String> records) {
 
         // This will ensure that we get the cluster metadata when onConsume is called for the first time
@@ -99,4 +100,4 @@ public class MockConsumerInterceptor implements ClusterResourceListener, Consume
     public void onUpdate(ClusterResource clusterResource) {
         CLUSTER_META.set(clusterResource);
     }
-}
\ No newline at end of file
+}
