@@ -1,20 +1,19 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
+ * contributor license agreements. See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
- * <p/>
- * http://www.apache.org/licenses/LICENSE-2.0
- * <p/>
+ * the License. You may obtain a copy of the License at
+ *
+ *    http://www.apache.org/licenses/LICENSE-2.0
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- **/
-
+ */
 package org.apache.kafka.connect.runtime;
 
 import org.apache.kafka.connect.errors.ConnectException;
@@ -55,6 +54,10 @@ public class SourceTaskOffsetCommitterTest extends ThreadedTest {
     private ConcurrentHashMap committers;
     @Mock
     private Logger mockLog;
+    @Mock private ScheduledFuture commitFuture;
+    @Mock private ScheduledFuture taskFuture;
+    @Mock private ConnectorTaskId taskId;
+    @Mock private WorkerSourceTask task;
 
     private SourceTaskOffsetCommitter committer;
 
@@ -82,15 +85,11 @@ public class SourceTaskOffsetCommitterTest extends ThreadedTest {
     public void testSchedule() throws Exception {
         Capture<Runnable> taskWrapper = EasyMock.newCapture();
 
-        ScheduledFuture commitFuture = PowerMock.createMock(ScheduledFuture.class);
         EasyMock.expect(executor.scheduleWithFixedDelay(
                 EasyMock.capture(taskWrapper), eq(DEFAULT_OFFSET_COMMIT_INTERVAL_MS),
                 eq(DEFAULT_OFFSET_COMMIT_INTERVAL_MS), eq(TimeUnit.MILLISECONDS))
         ).andReturn(commitFuture);
 
-        ConnectorTaskId taskId = PowerMock.createMock(ConnectorTaskId.class);
-        WorkerSourceTask task = PowerMock.createMock(WorkerSourceTask.class);
-
         EasyMock.expect(committers.put(taskId, commitFuture)).andReturn(null);
 
         PowerMock.replayAll();
@@ -136,9 +135,6 @@ public class SourceTaskOffsetCommitterTest extends ThreadedTest {
 
     @Test
     public void testRemove() throws Exception {
-        ConnectorTaskId taskId = PowerMock.createMock(ConnectorTaskId.class);
-        ScheduledFuture task = PowerMock.createMock(ScheduledFuture.class);
-
         // Try to remove a non-existing task
         EasyMock.expect(committers.remove(taskId)).andReturn(null);
         PowerMock.replayAll();
@@ -149,10 +145,10 @@ public class SourceTaskOffsetCommitterTest extends ThreadedTest {
         PowerMock.resetAll();
 
         // Try to remove an existing task
-        EasyMock.expect(committers.remove(taskId)).andReturn(task);
-        EasyMock.expect(task.cancel(eq(false))).andReturn(false);
-        EasyMock.expect(task.isDone()).andReturn(false);
-        EasyMock.expect(task.get()).andReturn(null);
+        EasyMock.expect(committers.remove(taskId)).andReturn(taskFuture);
+        EasyMock.expect(taskFuture.cancel(eq(false))).andReturn(false);
+        EasyMock.expect(taskFuture.isDone()).andReturn(false);
+        EasyMock.expect(taskFuture.get()).andReturn(null);
         PowerMock.replayAll();
 
         committer.remove(taskId);
@@ -161,10 +157,10 @@ public class SourceTaskOffsetCommitterTest extends ThreadedTest {
         PowerMock.resetAll();
 
         // Try to remove a cancelled task
-        EasyMock.expect(committers.remove(taskId)).andReturn(task);
-        EasyMock.expect(task.cancel(eq(false))).andReturn(false);
-        EasyMock.expect(task.isDone()).andReturn(false);
-        EasyMock.expect(task.get()).andThrow(new CancellationException());
+        EasyMock.expect(committers.remove(taskId)).andReturn(taskFuture);
+        EasyMock.expect(taskFuture.cancel(eq(false))).andReturn(false);
+        EasyMock.expect(taskFuture.isDone()).andReturn(false);
+        EasyMock.expect(taskFuture.get()).andThrow(new CancellationException());
         mockLog.trace(EasyMock.anyString(), EasyMock.<Object>anyObject());
         PowerMock.expectLastCall();
         PowerMock.replayAll();
@@ -175,10 +171,10 @@ public class SourceTaskOffsetCommitterTest extends ThreadedTest {
         PowerMock.resetAll();
 
         // Try to remove an interrupted task
-        EasyMock.expect(committers.remove(taskId)).andReturn(task);
-        EasyMock.expect(task.cancel(eq(false))).andReturn(false);
-        EasyMock.expect(task.isDone()).andReturn(false);
-        EasyMock.expect(task.get()).andThrow(new InterruptedException());
+        EasyMock.expect(committers.remove(taskId)).andReturn(taskFuture);
+        EasyMock.expect(taskFuture.cancel(eq(false))).andReturn(false);
+        EasyMock.expect(taskFuture.isDone()).andReturn(false);
+        EasyMock.expect(taskFuture.get()).andThrow(new InterruptedException());
         PowerMock.replayAll();
 
         try {
