@@ -1,13 +1,13 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
+ * contributor license agreements. See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
- * <p>
- * http://www.apache.org/licenses/LICENSE-2.0
- * <p>
+ * the License. You may obtain a copy of the License at
+ *
+ *    http://www.apache.org/licenses/LICENSE-2.0
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -18,6 +18,9 @@ package org.apache.kafka.streams.processor.internals;
 
 import org.apache.kafka.clients.consumer.ConsumerRecord;
 import org.apache.kafka.common.TopicPartition;
+import org.apache.kafka.common.utils.LogContext;
+import org.apache.kafka.streams.errors.DeserializationExceptionHandler;
+import org.apache.kafka.streams.errors.StreamsException;
 
 import java.io.IOException;
 import java.util.HashMap;
@@ -29,66 +32,66 @@ import java.util.Set;
  */
 public class GlobalStateUpdateTask implements GlobalStateMaintainer {
 
-    private class SourceNodeAndDeserializer {
-        private final SourceNode sourceNode;
-        private final RecordDeserializer deserializer;
-
-        SourceNodeAndDeserializer(final SourceNode sourceNode,
-                                  final RecordDeserializer deserializer) {
-            this.sourceNode = sourceNode;
-            this.deserializer = deserializer;
-        }
-    }
-
     private final ProcessorTopology topology;
     private final InternalProcessorContext processorContext;
     private final Map<TopicPartition, Long> offsets = new HashMap<>();
-    private final Map<String, SourceNodeAndDeserializer> deserializers = new HashMap<>();
+    private final Map<String, RecordDeserializer> deserializers = new HashMap<>();
     private final GlobalStateManager stateMgr;
-
+    private final DeserializationExceptionHandler deserializationExceptionHandler;
+    private final LogContext logContext;
 
     public GlobalStateUpdateTask(final ProcessorTopology topology,
                                  final InternalProcessorContext processorContext,
-                                 final GlobalStateManager stateMgr) {
-
+                                 final GlobalStateManager stateMgr,
+                                 final DeserializationExceptionHandler deserializationExceptionHandler,
+                                 final LogContext logContext) {
         this.topology = topology;
         this.stateMgr = stateMgr;
         this.processorContext = processorContext;
+        this.deserializationExceptionHandler = deserializationExceptionHandler;
+        this.logContext = logContext;
     }
 
-    @SuppressWarnings("unchecked")
+    /**
+     * @throws IllegalStateException If store gets registered after initialized is already finished
+     * @throws StreamsException if the store's change log does not contain the partition
+     */
+    @Override
     public Map<TopicPartition, Long> initialize() {
-        final Set<String> storeNames = stateMgr.initialize(processorContext);
+        final Set<String> storeNames = stateMgr.initialize();
         final Map<String, String> storeNameToTopic = topology.storeToChangelogTopic();
         for (final String storeName : storeNames) {
             final String sourceTopic = storeNameToTopic.get(storeName);
             final SourceNode source = topology.source(sourceTopic);
-            deserializers.put(sourceTopic, new SourceNodeAndDeserializer(source, new SourceNodeRecordDeserializer(source)));
+            deserializers.put(sourceTopic, new RecordDeserializer(source, deserializationExceptionHandler, logContext));
         }
         initTopology();
         processorContext.initialized();
         return stateMgr.checkpointed();
     }
 
-
     @SuppressWarnings("unchecked")
     @Override
     public void update(final ConsumerRecord<byte[], byte[]> record) {
-        final SourceNodeAndDeserializer sourceNodeAndDeserializer = deserializers.get(record.topic());
-        final ConsumerRecord<Object, Object> deserialized = sourceNodeAndDeserializer.deserializer.deserialize(record);
-        final ProcessorRecordContext recordContext =
+        final RecordDeserializer sourceNodeAndDeserializer = deserializers.get(record.topic());
+        final ConsumerRecord<Object, Object> deserialized = sourceNodeAndDeserializer.deserialize(processorContext, record);
+
+        if (deserialized != null) {
+            final ProcessorRecordContext recordContext =
                 new ProcessorRecordContext(deserialized.timestamp(),
-                                           deserialized.offset(),
-                                           deserialized.partition(),
-                                           deserialized.topic());
-        processorContext.setRecordContext(recordContext);
-        processorContext.setCurrentNode(sourceNodeAndDeserializer.sourceNode);
-        sourceNodeAndDeserializer.sourceNode.process(deserialized.key(), deserialized.value());
-        offsets.put(new TopicPartition(record.topic(), record.partition()), deserialized.offset() + 1);
+                    deserialized.offset(),
+                    deserialized.partition(),
+                    deserialized.topic());
+            processorContext.setRecordContext(recordContext);
+            processorContext.setCurrentNode(sourceNodeAndDeserializer.sourceNode());
+            sourceNodeAndDeserializer.sourceNode().process(deserialized.key(), deserialized.value());
+        }
+
+        offsets.put(new TopicPartition(record.topic(), record.partition()), record.offset() + 1);
     }
 
     public void flushState() {
-        stateMgr.flush(processorContext);
+        stateMgr.flush();
         stateMgr.checkpoint(offsets);
     }
 
