@@ -1,10 +1,10 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
+ * contributor license agreements. See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
+ * the License. You may obtain a copy of the License at
  *
  *    http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -14,7 +14,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.kafka.streams.processor.internals;
 
 import org.apache.kafka.common.serialization.Serializer;
@@ -32,7 +31,11 @@ public class SinkNode<K, V> extends ProcessorNode<K, V> {
 
     private ProcessorContext context;
 
-    public SinkNode(String name, String topic, Serializer<K> keySerializer, Serializer<V> valSerializer, StreamPartitioner<? super K, ? super V> partitioner) {
+    public SinkNode(final String name,
+                    final String topic,
+                    final Serializer<K> keySerializer,
+                    final Serializer<V> valSerializer,
+                    final StreamPartitioner<? super K, ? super V> partitioner) {
         super(name);
 
         this.topic = topic;
@@ -45,30 +48,35 @@ public class SinkNode<K, V> extends ProcessorNode<K, V> {
      * @throws UnsupportedOperationException if this method adds a child to a sink node
      */
     @Override
-    public void addChild(ProcessorNode<?, ?> child) {
+    public void addChild(final ProcessorNode<?, ?> child) {
         throw new UnsupportedOperationException("sink node does not allow addChild");
     }
 
     @SuppressWarnings("unchecked")
     @Override
-    public void init(ProcessorContext context) {
+    public void init(final ProcessorContext context) {
         super.init(context);
         this.context = context;
 
         // if serializers are null, get the default ones from the context
-        if (this.keySerializer == null) this.keySerializer = (Serializer<K>) context.keySerde().serializer();
-        if (this.valSerializer == null) this.valSerializer = (Serializer<V>) context.valueSerde().serializer();
+        if (keySerializer == null) {
+            keySerializer = (Serializer<K>) context.keySerde().serializer();
+        }
+        if (valSerializer == null) {
+            valSerializer = (Serializer<V>) context.valueSerde().serializer();
+        }
 
         // if value serializers are for {@code Change} values, set the inner serializer when necessary
-        if (this.valSerializer instanceof ChangedSerializer &&
-                ((ChangedSerializer) this.valSerializer).inner() == null)
-            ((ChangedSerializer) this.valSerializer).setInner(context.valueSerde().serializer());
+        if (valSerializer instanceof ChangedSerializer &&
+                ((ChangedSerializer) valSerializer).inner() == null) {
+            ((ChangedSerializer) valSerializer).setInner(context.valueSerde().serializer());
+        }
     }
 
 
     @Override
     public void process(final K key, final V value) {
-        RecordCollector collector = ((RecordCollector.Supplier) context).recordCollector();
+        final RecordCollector collector = ((RecordCollector.Supplier) context).recordCollector();
 
         final long timestamp = context.timestamp();
         if (timestamp < 0) {
@@ -76,16 +84,18 @@ public class SinkNode<K, V> extends ProcessorNode<K, V> {
         }
 
         try {
-            collector.send(topic, key, value, null, timestamp, keySerializer, valSerializer, partitioner);
-        } catch (ClassCastException e) {
+            collector.send(topic, key, value, timestamp, keySerializer, valSerializer, partitioner);
+        } catch (final ClassCastException e) {
+            final String keyClass = key == null ? "unknown because key is null" : key.getClass().getName();
+            final String valueClass = value == null ? "unknown because value is null" : value.getClass().getName();
             throw new StreamsException(
                     String.format("A serializer (key: %s / value: %s) is not compatible to the actual key or value type " +
                                     "(key type: %s / value type: %s). Change the default Serdes in StreamConfig or " +
                                     "provide correct Serdes via method parameters.",
                                     keySerializer.getClass().getName(),
                                     valSerializer.getClass().getName(),
-                                    key.getClass().getName(),
-                                    value.getClass().getName()),
+                                    keyClass,
+                                    valueClass),
                     e);
         }
     }
@@ -101,7 +111,8 @@ public class SinkNode<K, V> extends ProcessorNode<K, V> {
     /**
      * @return a string representation of this node starting with the given indent, useful for debugging.
      */
-    public String toString(String indent) {
+    @Override
+    public String toString(final String indent) {
         final StringBuilder sb = new StringBuilder(super.toString(indent));
         sb.append(indent).append("\ttopic:\t\t");
         sb.append(topic);
