@@ -1,10 +1,10 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
+ * contributor license agreements. See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
+ * the License. You may obtain a copy of the License at
  *
  *    http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -14,7 +14,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.kafka.streams.processor.internals.assignment;
 
 import org.apache.kafka.streams.processor.TaskId;
@@ -48,7 +47,7 @@ public class SubscriptionInfoTest {
     }
 
     @Test
-    public void shouldEncodeDecodeWithUserEndPoint() throws Exception {
+    public void shouldEncodeDecodeWithUserEndPoint() {
         SubscriptionInfo original = new SubscriptionInfo(UUID.randomUUID(),
                 Collections.singleton(new TaskId(0, 0)), Collections.<TaskId>emptySet(), "localhost:80");
         SubscriptionInfo decoded = SubscriptionInfo.decode(original.encode());
@@ -56,7 +55,7 @@ public class SubscriptionInfoTest {
     }
 
     @Test
-    public void shouldBeBackwardCompatible() throws Exception {
+    public void shouldBeBackwardCompatible() {
         UUID processId = UUID.randomUUID();
 
         Set<TaskId> activeTasks =
