@@ -1,13 +1,13 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
+ * contributor license agreements. See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
- * <p>
- * http://www.apache.org/licenses/LICENSE-2.0
- * <p>
+ * the License. You may obtain a copy of the License at
+ *
+ *    http://www.apache.org/licenses/LICENSE-2.0
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -16,8 +16,29 @@
  */
 package org.apache.kafka.streams.state;
 
+import org.apache.kafka.clients.producer.MockProducer;
+import org.apache.kafka.clients.producer.Producer;
+import org.apache.kafka.common.serialization.ByteArraySerializer;
+import org.apache.kafka.common.serialization.Deserializer;
+import org.apache.kafka.common.serialization.Serdes;
+import org.apache.kafka.common.serialization.Serializer;
+import org.apache.kafka.common.utils.LogContext;
+import org.apache.kafka.streams.KeyValue;
+import org.apache.kafka.streams.StreamsConfig;
+import org.apache.kafka.streams.errors.DefaultProductionExceptionHandler;
+import org.apache.kafka.streams.processor.ProcessorContext;
+import org.apache.kafka.streams.processor.StateRestoreCallback;
+import org.apache.kafka.streams.processor.StateStore;
+import org.apache.kafka.streams.processor.StreamPartitioner;
+import org.apache.kafka.streams.processor.internals.RecordCollector;
+import org.apache.kafka.streams.processor.internals.RecordCollectorImpl;
+import org.apache.kafka.streams.state.internals.RocksDBKeyValueStoreTest;
+import org.apache.kafka.streams.state.internals.ThreadCache;
+import org.apache.kafka.test.MockProcessorContext;
+import org.apache.kafka.test.MockTimestampExtractor;
+import org.apache.kafka.test.TestUtils;
+
 import java.io.File;
-import java.util.Arrays;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.LinkedList;
@@ -26,33 +47,6 @@ import java.util.Map;
 import java.util.Objects;
 import java.util.Properties;
 import java.util.Set;
-import org.apache.kafka.clients.producer.MockProducer;
-import org.apache.kafka.clients.producer.Producer;
-import org.apache.kafka.common.metrics.JmxReporter;
-import org.apache.kafka.common.metrics.MetricConfig;
-import org.apache.kafka.common.metrics.Metrics;
-import org.apache.kafka.common.metrics.MetricsReporter;
-import org.apache.kafka.common.serialization.ByteArraySerializer;
-import org.apache.kafka.common.serialization.Deserializer;
-import org.apache.kafka.common.serialization.Serdes;
-import org.apache.kafka.common.serialization.Serializer;
-import org.apache.kafka.common.utils.MockTime;
-import org.apache.kafka.streams.KeyValue;
-import org.apache.kafka.streams.StreamsConfig;
-import org.apache.kafka.streams.StreamsMetrics;
-import org.apache.kafka.streams.processor.ProcessorContext;
-import org.apache.kafka.streams.processor.StateRestoreCallback;
-import org.apache.kafka.streams.processor.StateStore;
-import org.apache.kafka.streams.processor.StreamPartitioner;
-import org.apache.kafka.streams.processor.TaskId;
-import org.apache.kafka.streams.processor.internals.MockStreamsMetrics;
-import org.apache.kafka.streams.processor.internals.ProcessorNode;
-import org.apache.kafka.streams.processor.internals.RecordCollector;
-import org.apache.kafka.streams.processor.internals.RecordCollectorImpl;
-import org.apache.kafka.streams.state.internals.ThreadCache;
-import org.apache.kafka.test.MockProcessorContext;
-import org.apache.kafka.test.MockTimestampExtractor;
-import org.apache.kafka.test.TestUtils;
 
 /**
  * A component that provides a {@link #context() ProcessingContext} that can be supplied to a {@link KeyValueStore} so that
@@ -153,9 +147,9 @@ public class KeyValueStoreTestDriver<K, V> {
      *            {@code Long.class}, or {@code byte[].class}
      * @return the test driver; never null
      */
-    public static <K, V> KeyValueStoreTestDriver<K, V> create(Class<K> keyClass, Class<V> valueClass) {
-        StateSerdes<K, V> serdes = StateSerdes.withBuiltinTypes("unexpected", keyClass, valueClass);
-        return new KeyValueStoreTestDriver<K, V>(serdes);
+    public static <K, V> KeyValueStoreTestDriver<K, V> create(final Class<K> keyClass, final Class<V> valueClass) {
+        final StateSerdes<K, V> serdes = StateSerdes.withBuiltinTypes("unexpected", keyClass, valueClass);
+        return new KeyValueStoreTestDriver<>(serdes);
     }
 
     /**
@@ -174,104 +168,71 @@ public class KeyValueStoreTestDriver<K, V> {
                                                               final Deserializer<K> keyDeserializer,
                                                               final Serializer<V> valueSerializer,
                                                               final Deserializer<V> valueDeserializer) {
-        StateSerdes<K, V> serdes = new StateSerdes<K, V>("unexpected",
+        final StateSerdes<K, V> serdes = new StateSerdes<>(
+            "unexpected",
             Serdes.serdeFrom(keySerializer, keyDeserializer),
             Serdes.serdeFrom(valueSerializer, valueDeserializer));
-        return new KeyValueStoreTestDriver<K, V>(serdes);
+        return new KeyValueStoreTestDriver<>(serdes);
     }
 
     private final Map<K, V> flushedEntries = new HashMap<>();
     private final Set<K> flushedRemovals = new HashSet<>();
-    private final List<KeyValue<K, V>> restorableEntries = new LinkedList<>();
+    private final List<KeyValue<byte[], byte[]>> restorableEntries = new LinkedList<>();
+
     private final MockProcessorContext context;
-    private final Map<String, StateStore> storeMap = new HashMap<>();
-    private MockTime time = new MockTime();
-    private MetricConfig config = new MetricConfig();
-    private Metrics metrics = new Metrics(config, Arrays.asList((MetricsReporter) new JmxReporter()), time, true);
+    private final StateSerdes<K, V> stateSerdes;
 
-    private static final long DEFAULT_CACHE_SIZE_BYTES = 1 * 1024 * 1024L;
-    private final ThreadCache cache = new ThreadCache("testCache", DEFAULT_CACHE_SIZE_BYTES, new MockStreamsMetrics(new Metrics()));
-    private final StreamsMetrics streamsMetrics = new MockStreamsMetrics(metrics);
-    private final RecordCollector recordCollector;
-    private File stateDir = null;
+    private KeyValueStoreTestDriver(final StateSerdes<K, V> serdes) {
+        final ByteArraySerializer rawSerializer = new ByteArraySerializer();
+        final Producer<byte[], byte[]> producer = new MockProducer<>(true, rawSerializer, rawSerializer);
 
-    protected KeyValueStoreTestDriver(final StateSerdes<K, V> serdes) {
-        ByteArraySerializer rawSerializer = new ByteArraySerializer();
-        Producer<byte[], byte[]> producer = new MockProducer<>(true, rawSerializer, rawSerializer);
-
-        this.recordCollector = new RecordCollectorImpl(producer, "KeyValueStoreTestDriver") {
-            @SuppressWarnings("unchecked")
+        final RecordCollector recordCollector = new RecordCollectorImpl(producer, "KeyValueStoreTestDriver", new LogContext("KeyValueStoreTestDriver "), new DefaultProductionExceptionHandler()) {
             @Override
             public <K1, V1> void send(final String topic,
-                                      K1 key,
-                                      V1 value,
-                                      Integer partition,
-                                      Long timestamp,
-                                      Serializer<K1> keySerializer,
-                                      Serializer<V1> valueSerializer) {
+                                      final K1 key,
+                                      final V1 value,
+                                      final Integer partition,
+                                      final Long timestamp,
+                                      final Serializer<K1> keySerializer,
+                                      final Serializer<V1> valueSerializer) {
             // for byte arrays we need to wrap it for comparison
 
-                K keyTest = serdes.keyFrom(keySerializer.serialize(topic, key));
-                V valueTest = serdes.valueFrom(valueSerializer.serialize(topic, value));
+                final K keyTest = serdes.keyFrom(keySerializer.serialize(topic, key));
+                final V valueTest = serdes.valueFrom(valueSerializer.serialize(topic, value));
 
                 recordFlushed(keyTest, valueTest);
             }
 
             @Override
             public <K1, V1> void send(final String topic,
-                                      K1 key,
-                                      V1 value,
-                                      Integer partition,
-                                      Long timestamp,
-                                      Serializer<K1> keySerializer,
-                                      Serializer<V1> valueSerializer,
-                                      StreamPartitioner<? super K1, ? super V1> partitioner) {
-                // ignore partitioner
-                send(topic, key, value, partition, timestamp, keySerializer, valueSerializer);
+                                      final K1 key,
+                                      final V1 value,
+                                      final Long timestamp,
+                                      final Serializer<K1> keySerializer,
+                                      final Serializer<V1> valueSerializer,
+                                      final StreamPartitioner<? super K1, ? super V1> partitioner) {
+                throw new UnsupportedOperationException();
             }
         };
-        this.stateDir = TestUtils.tempDirectory();
-        this.stateDir.mkdirs();
+
+        File stateDir = TestUtils.tempDirectory();
+        stateDir.mkdirs();
+        stateSerdes = serdes;
 
         props = new Properties();
-        props.put(StreamsConfig.APPLICATION_ID_CONFIG, "applicationId");
+        props.put(StreamsConfig.APPLICATION_ID_CONFIG, "application-id");
         props.put(StreamsConfig.BOOTSTRAP_SERVERS_CONFIG, "localhost:9092");
-        props.put(StreamsConfig.TIMESTAMP_EXTRACTOR_CLASS_CONFIG, MockTimestampExtractor.class);
-        props.put(StreamsConfig.KEY_SERDE_CLASS_CONFIG, serdes.keySerde().getClass());
-        props.put(StreamsConfig.VALUE_SERDE_CLASS_CONFIG, serdes.valueSerde().getClass());
+        props.put(StreamsConfig.DEFAULT_TIMESTAMP_EXTRACTOR_CLASS_CONFIG, MockTimestampExtractor.class);
+        props.put(StreamsConfig.DEFAULT_KEY_SERDE_CLASS_CONFIG, serdes.keySerde().getClass());
+        props.put(StreamsConfig.DEFAULT_VALUE_SERDE_CLASS_CONFIG, serdes.valueSerde().getClass());
+        props.put(StreamsConfig.ROCKSDB_CONFIG_SETTER_CLASS_CONFIG, RocksDBKeyValueStoreTest.TheRocksDbConfigSetter.class);
 
-
-
-        this.context = new MockProcessorContext(this.stateDir, serdes.keySerde(), serdes.valueSerde(), recordCollector, null) {
-            @Override
-            public TaskId taskId() {
-                return new TaskId(0, 1);
-            }
+        context = new MockProcessorContext(stateDir, serdes.keySerde(), serdes.valueSerde(), recordCollector, null) {
+            ThreadCache cache = new ThreadCache(new LogContext("testCache "), 1024 * 1024L, metrics());
 
             @Override
-            public <K1, V1> void forward(K1 key, V1 value, int childIndex) {
-                forward(key, value);
-            }
-
-            @Override
-            public void register(StateStore store, boolean loggingEnabled, StateRestoreCallback func) {
-                storeMap.put(store.name(), store);
-                restoreEntries(func, serdes);
-            }
-
-            @Override
-            public StateStore getStateStore(String name) {
-                return storeMap.get(name);
-            }
-
-            @Override
-            public StreamsMetrics metrics() {
-                return streamsMetrics;
-            }
-
-            @Override
-            public File stateDir() {
-                return stateDir;
+            public ThreadCache getCache() {
+                return cache;
             }
 
             @Override
@@ -280,23 +241,13 @@ public class KeyValueStoreTestDriver<K, V> {
             }
 
             @Override
-            public Map<String, Object> appConfigsWithPrefix(String prefix) {
+            public Map<String, Object> appConfigsWithPrefix(final String prefix) {
                 return new StreamsConfig(props).originalsWithPrefix(prefix);
             }
-
-            @Override
-            public ProcessorNode currentNode() {
-                return null;
-            }
-
-            @Override
-            public ThreadCache getCache() {
-                return cache;
-            }
         };
     }
 
-    protected void recordFlushed(K key, V value) {
+    private void recordFlushed(final K key, final V value) {
         if (value == null) {
             // This is a removal ...
             flushedRemovals.add(key);
@@ -308,14 +259,14 @@ public class KeyValueStoreTestDriver<K, V> {
         }
     }
 
-    private void restoreEntries(StateRestoreCallback func, StateSerdes<K, V> serdes) {
-        for (KeyValue<K, V> entry : restorableEntries) {
-            if (entry != null) {
-                byte[] rawKey = serdes.rawKey(entry.key);
-                byte[] rawValue = serdes.rawValue(entry.value);
-                func.restore(rawKey, rawValue);
-            }
-        }
+    /**
+     * Get the entries that are restored to a KeyValueStore when it is constructed with this driver's {@link #context()
+     * ProcessorContext}.
+     *
+     * @return the restore entries; never null but possibly a null iterator
+     */
+    public Iterable<KeyValue<byte[], byte[]>> restoredEntries() {
+        return restorableEntries;
     }
 
     /**
@@ -347,8 +298,8 @@ public class KeyValueStoreTestDriver<K, V> {
      * @param value the value for the entry
      * @see #checkForRestoredEntries(KeyValueStore)
      */
-    public void addEntryToRestoreLog(K key, V value) {
-        restorableEntries.add(new KeyValue<K, V>(key, value));
+    public void addEntryToRestoreLog(final K key, final V value) {
+        restorableEntries.add(new KeyValue<>(stateSerdes.rawKey(key), stateSerdes.rawValue(value)));
     }
 
     /**
@@ -366,16 +317,6 @@ public class KeyValueStoreTestDriver<K, V> {
         return context;
     }
 
-    /**
-     * Get the entries that are restored to a KeyValueStore when it is constructed with this driver's {@link #context()
-     * ProcessorContext}.
-     *
-     * @return the restore entries; never null but possibly a null iterator
-     */
-    public Iterable<KeyValue<K, V>> restoredEntries() {
-        return restorableEntries;
-    }
-
     /**
      * Utility method that will count the number of {@link #addEntryToRestoreLog(Object, Object) restore entries} missing from the
      * supplied store.
@@ -384,12 +325,12 @@ public class KeyValueStoreTestDriver<K, V> {
      * @return the number of restore entries missing from the store, or 0 if all restore entries were found
      * @see #addEntryToRestoreLog(Object, Object)
      */
-    public int checkForRestoredEntries(KeyValueStore<K, V> store) {
+    public int checkForRestoredEntries(final KeyValueStore<K, V> store) {
         int missing = 0;
-        for (KeyValue<K, V> kv : restorableEntries) {
+        for (final KeyValue<byte[], byte[]> kv : restorableEntries) {
             if (kv != null) {
-                V value = store.get(kv.key);
-                if (!Objects.equals(value, kv.value)) {
+                final V value = store.get(stateSerdes.keyFrom(kv.key));
+                if (!Objects.equals(value, stateSerdes.valueFrom(kv.value))) {
                     ++missing;
                 }
             }
@@ -403,7 +344,7 @@ public class KeyValueStoreTestDriver<K, V> {
      * @param store the key value store using this {@link #context()}.
      * @return the number of entries
      */
-    public int sizeOf(KeyValueStore<K, V> store) {
+    public int sizeOf(final KeyValueStore<K, V> store) {
         int size = 0;
         try (KeyValueIterator<K, V> iterator = store.all()) {
             while (iterator.hasNext()) {
@@ -419,9 +360,9 @@ public class KeyValueStoreTestDriver<K, V> {
      *
      * @param key the key
      * @return the value that was flushed with the key, or {@code null} if no such key was flushed or if the entry with this
-     *         key was {@link #flushedEntryStored(Object) removed} upon flush
+     *         key was removed upon flush
      */
-    public V flushedEntryStored(K key) {
+    public V flushedEntryStored(final K key) {
         return flushedEntries.get(key);
     }
 
@@ -432,10 +373,17 @@ public class KeyValueStoreTestDriver<K, V> {
      * @return {@code true} if the entry with the given key was removed when flushed, or {@code false} if the entry was not
      *         removed when last flushed
      */
-    public boolean flushedEntryRemoved(K key) {
+    public boolean flushedEntryRemoved(final K key) {
         return flushedRemovals.contains(key);
     }
 
+    /**
+     * Return number of removed entry
+     */
+    public int numFlushedEntryStored() {
+        return flushedEntries.size();
+    }
+
     /**
      * Return number of removed entry
      */
@@ -451,8 +399,4 @@ public class KeyValueStoreTestDriver<K, V> {
         flushedEntries.clear();
         flushedRemovals.clear();
     }
-
-    public void setConfig(final String configName, final Object configValue) {
-        props.put(configName, configValue);
-    }
 }
