@@ -1,20 +1,19 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
+ * contributor license agreements. See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
- * <p>
- * http://www.apache.org/licenses/LICENSE-2.0
- * <p>
+ * the License. You may obtain a copy of the License at
+ *
+ *    http://www.apache.org/licenses/LICENSE-2.0
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.kafka.streams.state.internals;
 
 import org.apache.kafka.common.metrics.Metrics;
@@ -43,11 +42,12 @@ public class NamedCacheTest {
 
     private NamedCache cache;
     private MockStreamsMetrics streamMetrics;
-
+    private final String taskIDString = "0.0";
+    private final String underlyingStoreName = "storeName";
     @Before
-    public void setUp() throws Exception {
+    public void setUp() {
         streamMetrics = new MockStreamsMetrics(new Metrics());
-        cache = new NamedCache("name", streamMetrics);
+        cache = new NamedCache(taskIDString + "-" + underlyingStoreName, streamMetrics);
     }
 
     @Test
@@ -73,32 +73,37 @@ public class NamedCacheTest {
         }
     }
 
+    private void testSpecificMetrics(final String groupName, final String entityName, final String opName,
+                                     final Map<String, String> metricTags) {
+        assertNotNull(streamMetrics.registry().metrics().get(streamMetrics.registry().metricName(opName + "-avg",
+                groupName, "The average cache hit ratio of " + entityName, metricTags)));
+        assertNotNull(streamMetrics.registry().metrics().get(streamMetrics.registry().metricName(opName + "-min",
+                groupName, "The minimum cache hit ratio of " + entityName, metricTags)));
+        assertNotNull(streamMetrics.registry().metrics().get(streamMetrics.registry().metricName(opName + "-max",
+                groupName, "The maximum cache hit ratio of " + entityName, metricTags)));
+    }
     @Test
-    public void testMetrics() throws Exception {
+    public void testMetrics() {
         final String scope = "record-cache";
         final String entityName = cache.name();
         final String opName = "hitRatio";
         final String tagKey = "record-cache-id";
-        final String tagValue = cache.name();
+        final String tagValue = underlyingStoreName;
         final String groupName = "stream-" + scope + "-metrics";
         final Map<String, String> metricTags = new LinkedHashMap<>();
         metricTags.put(tagKey, tagValue);
+        metricTags.put("task-id", taskIDString);
 
-        assertNotNull(streamMetrics.registry().getSensor(entityName + "-" + opName));
-        assertNotNull(streamMetrics.registry().metrics().get(streamMetrics.registry().metricName(entityName +
-            "-" + opName + "-avg", groupName, "The current count of " + entityName + " " + opName +
-            " operation.", metricTags)));
-        assertNotNull(streamMetrics.registry().metrics().get(streamMetrics.registry().metricName(entityName +
-            "-" + opName + "-min", groupName, "The current count of " + entityName + " " + opName +
-            " operation.", metricTags)));
-        assertNotNull(streamMetrics.registry().metrics().get(streamMetrics.registry().metricName(entityName +
-            "-" + opName + "-max", groupName, "The current count of " + entityName + " " + opName +
-            " operation.", metricTags)));
+        assertNotNull(streamMetrics.registry().getSensor(opName));
+        testSpecificMetrics(groupName, entityName, opName, metricTags);
 
+        // test "all"
+        metricTags.put(tagKey, "all");
+        testSpecificMetrics(groupName, entityName, opName, metricTags);
     }
 
     @Test
-    public void shouldKeepTrackOfSize() throws Exception {
+    public void shouldKeepTrackOfSize() {
         final LRUCacheEntry value = new LRUCacheEntry(new byte[]{0});
         cache.put(Bytes.wrap(new byte[]{0}), value);
         cache.put(Bytes.wrap(new byte[]{1}), value);
@@ -109,7 +114,7 @@ public class NamedCacheTest {
     }
 
     @Test
-    public void shouldPutGet() throws Exception {
+    public void shouldPutGet() {
         cache.put(Bytes.wrap(new byte[]{0}), new LRUCacheEntry(new byte[]{10}));
         cache.put(Bytes.wrap(new byte[]{1}), new LRUCacheEntry(new byte[]{11}));
         cache.put(Bytes.wrap(new byte[]{2}), new LRUCacheEntry(new byte[]{12}));
@@ -121,7 +126,7 @@ public class NamedCacheTest {
     }
 
     @Test
-    public void shouldPutIfAbsent() throws Exception {
+    public void shouldPutIfAbsent() {
         cache.put(Bytes.wrap(new byte[]{0}), new LRUCacheEntry(new byte[]{10}));
         cache.putIfAbsent(Bytes.wrap(new byte[]{0}), new LRUCacheEntry(new byte[]{20}));
         cache.putIfAbsent(Bytes.wrap(new byte[]{1}), new LRUCacheEntry(new byte[]{30}));
@@ -131,7 +136,7 @@ public class NamedCacheTest {
     }
 
     @Test
-    public void shouldDeleteAndUpdateSize() throws Exception {
+    public void shouldDeleteAndUpdateSize() {
         cache.put(Bytes.wrap(new byte[]{0}), new LRUCacheEntry(new byte[]{10}));
         final LRUCacheEntry deleted = cache.delete(Bytes.wrap(new byte[]{0}));
         assertArrayEquals(new byte[] {10}, deleted.value);
@@ -139,7 +144,7 @@ public class NamedCacheTest {
     }
 
     @Test
-    public void shouldPutAll() throws Exception {
+    public void shouldPutAll() {
         cache.putAll(Arrays.asList(KeyValue.pair(new byte[] {0}, new LRUCacheEntry(new byte[]{0})),
                                    KeyValue.pair(new byte[] {1}, new LRUCacheEntry(new byte[]{1})),
                                    KeyValue.pair(new byte[] {2}, new LRUCacheEntry(new byte[]{2}))));
@@ -150,7 +155,7 @@ public class NamedCacheTest {
     }
 
     @Test
-    public void shouldOverwriteAll() throws Exception {
+    public void shouldOverwriteAll() {
         cache.putAll(Arrays.asList(KeyValue.pair(new byte[] {0}, new LRUCacheEntry(new byte[]{0})),
             KeyValue.pair(new byte[] {0}, new LRUCacheEntry(new byte[]{1})),
             KeyValue.pair(new byte[] {0}, new LRUCacheEntry(new byte[]{2}))));
@@ -160,7 +165,7 @@ public class NamedCacheTest {
     }
 
     @Test
-    public void shouldEvictEldestEntry() throws Exception {
+    public void shouldEvictEldestEntry() {
         cache.put(Bytes.wrap(new byte[]{0}), new LRUCacheEntry(new byte[]{10}));
         cache.put(Bytes.wrap(new byte[]{1}), new LRUCacheEntry(new byte[]{20}));
         cache.put(Bytes.wrap(new byte[]{2}), new LRUCacheEntry(new byte[]{30}));
@@ -171,7 +176,7 @@ public class NamedCacheTest {
     }
 
     @Test
-    public void shouldFlushDirtEntriesOnEviction() throws Exception {
+    public void shouldFlushDirtEntriesOnEviction() {
         final List<ThreadCache.DirtyEntry> flushed = new ArrayList<>();
         cache.put(Bytes.wrap(new byte[]{0}), new LRUCacheEntry(new byte[]{10}, true, 0, 0, 0, ""));
         cache.put(Bytes.wrap(new byte[]{1}), new LRUCacheEntry(new byte[]{20}));
@@ -195,7 +200,7 @@ public class NamedCacheTest {
     }
 
     @Test
-    public void shouldGetRangeIteratorOverKeys() throws Exception {
+    public void shouldGetRangeIteratorOverKeys() {
         cache.put(Bytes.wrap(new byte[]{0}), new LRUCacheEntry(new byte[]{10}, true, 0, 0, 0, ""));
         cache.put(Bytes.wrap(new byte[]{1}), new LRUCacheEntry(new byte[]{20}));
         cache.put(Bytes.wrap(new byte[]{2}), new LRUCacheEntry(new byte[]{30}, true, 0, 0, 0, ""));
@@ -207,7 +212,7 @@ public class NamedCacheTest {
     }
 
     @Test
-    public void shouldGetIteratorOverAllKeys() throws Exception {
+    public void shouldGetIteratorOverAllKeys() {
         cache.put(Bytes.wrap(new byte[]{0}), new LRUCacheEntry(new byte[]{10}, true, 0, 0, 0, ""));
         cache.put(Bytes.wrap(new byte[]{1}), new LRUCacheEntry(new byte[]{20}));
         cache.put(Bytes.wrap(new byte[]{2}), new LRUCacheEntry(new byte[]{30}, true, 0, 0, 0, ""));
@@ -220,18 +225,18 @@ public class NamedCacheTest {
     }
 
     @Test
-    public void shouldNotThrowNullPointerWhenCacheIsEmptyAndEvictionCalled() throws Exception {
+    public void shouldNotThrowNullPointerWhenCacheIsEmptyAndEvictionCalled() {
         cache.evict();
     }
 
     @Test(expected = IllegalStateException.class)
-    public void shouldThrowIllegalStateExceptionWhenTryingToOverwriteDirtyEntryWithCleanEntry() throws Exception {
+    public void shouldThrowIllegalStateExceptionWhenTryingToOverwriteDirtyEntryWithCleanEntry() {
         cache.put(Bytes.wrap(new byte[]{0}), new LRUCacheEntry(new byte[]{10}, true, 0, 0, 0, ""));
         cache.put(Bytes.wrap(new byte[]{0}), new LRUCacheEntry(new byte[]{10}, false, 0, 0, 0, ""));
     }
 
     @Test
-    public void shouldRemoveDeletedValuesOnFlush() throws Exception {
+    public void shouldRemoveDeletedValuesOnFlush() {
         cache.setListener(new ThreadCache.DirtyEntryFlushListener() {
             @Override
             public void apply(final List<ThreadCache.DirtyEntry> dirty) {
@@ -246,7 +251,7 @@ public class NamedCacheTest {
     }
 
     @Test
-    public void shouldBeReentrantAndNotBreakLRU() throws Exception {
+    public void shouldBeReentrantAndNotBreakLRU() {
         final LRUCacheEntry dirty = new LRUCacheEntry(new byte[]{3}, true, 0, 0, 0, "");
         final LRUCacheEntry clean = new LRUCacheEntry(new byte[]{3});
         cache.put(Bytes.wrap(new byte[]{0}), dirty);
@@ -292,7 +297,7 @@ public class NamedCacheTest {
     }
 
     @Test
-    public void shouldNotThrowIllegalArgumentAfterEvictingDirtyRecordAndThenPuttingNewRecordWithSameKey() throws Exception {
+    public void shouldNotThrowIllegalArgumentAfterEvictingDirtyRecordAndThenPuttingNewRecordWithSameKey() {
         final LRUCacheEntry dirty = new LRUCacheEntry(new byte[]{3}, true, 0, 0, 0, "");
         final LRUCacheEntry clean = new LRUCacheEntry(new byte[]{3});
         final Bytes key = Bytes.wrap(new byte[] {3});
@@ -307,7 +312,7 @@ public class NamedCacheTest {
     }
 
     @Test
-    public void shouldReturnNullIfKeyIsNull() throws Exception {
+    public void shouldReturnNullIfKeyIsNull() {
         assertNull(cache.get(null));
     }
 }
\ No newline at end of file
