@@ -1,16 +1,18 @@
-/**
- * Licensed to the Apache Software Foundation (ASF) under one or more contributor license
- * agreements.  See the NOTICE file distributed with this work for additional information regarding
- * copyright ownership. The ASF licenses this file to You under the Apache License, Version 2.0 (the
- * "License"); you may not use this file except in compliance with the License.  You may obtain a
- * copy of the License at
+/*
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements. See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
  *
- * http://www.apache.org/licenses/LICENSE-2.0
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
- * Unless required by applicable law or agreed to in writing, software distributed under the License
- * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
- * or implied. See the License for the specific language governing permissions and limitations under
- * the License.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 package org.apache.kafka.streams.state.internals;
 
@@ -48,37 +50,37 @@ public class QueryableStoreProviderTest {
     }
 
     @Test(expected = InvalidStateStoreException.class)
-    public void shouldThrowExceptionIfKVStoreDoesntExist() throws Exception {
+    public void shouldThrowExceptionIfKVStoreDoesntExist() {
         storeProvider.getStore("not-a-store", QueryableStoreTypes.keyValueStore());
     }
 
     @Test(expected = InvalidStateStoreException.class)
-    public void shouldThrowExceptionIfWindowStoreDoesntExist() throws Exception {
+    public void shouldThrowExceptionIfWindowStoreDoesntExist() {
         storeProvider.getStore("not-a-store", QueryableStoreTypes.windowStore());
     }
 
     @Test
-    public void shouldReturnKVStoreWhenItExists() throws Exception {
+    public void shouldReturnKVStoreWhenItExists() {
         assertNotNull(storeProvider.getStore(keyValueStore, QueryableStoreTypes.keyValueStore()));
     }
 
     @Test
-    public void shouldReturnWindowStoreWhenItExists() throws Exception {
+    public void shouldReturnWindowStoreWhenItExists() {
         assertNotNull(storeProvider.getStore(windowStore, QueryableStoreTypes.windowStore()));
     }
 
     @Test(expected = InvalidStateStoreException.class)
-    public void shouldThrowExceptionWhenLookingForWindowStoreWithDifferentType() throws Exception {
+    public void shouldThrowExceptionWhenLookingForWindowStoreWithDifferentType() {
         storeProvider.getStore(windowStore, QueryableStoreTypes.keyValueStore());
     }
 
     @Test(expected = InvalidStateStoreException.class)
-    public void shouldThrowExceptionWhenLookingForKVStoreWithDifferentType() throws Exception {
+    public void shouldThrowExceptionWhenLookingForKVStoreWithDifferentType() {
         storeProvider.getStore(keyValueStore, QueryableStoreTypes.windowStore());
     }
 
     @Test
-    public void shouldFindGlobalStores() throws Exception {
+    public void shouldFindGlobalStores() {
         globalStateStores.put("global", new NoOpReadOnlyStore<>());
         assertNotNull(storeProvider.getStore("global", QueryableStoreTypes.keyValueStore()));
     }
