@@ -1,13 +1,13 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements. See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License. You may obtain a copy of the License at
- * <p>
- * http://www.apache.org/licenses/LICENSE-2.0
- * <p>
+ *
+ *    http://www.apache.org/licenses/LICENSE-2.0
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -19,6 +19,7 @@ package org.apache.kafka.streams.state.internals;
 
 import org.apache.kafka.common.metrics.Metrics;
 import org.apache.kafka.common.utils.Bytes;
+import org.apache.kafka.common.utils.LogContext;
 import org.apache.kafka.streams.KeyValue;
 import org.apache.kafka.streams.processor.internals.MockStreamsMetrics;
 import org.junit.Test;
@@ -37,6 +38,10 @@ import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
 
 public class ThreadCacheTest {
+    final String namespace = "0.0-namespace";
+    final String namespace1 = "0.1-namespace";
+    final String namespace2 = "0.2-namespace";
+    private final LogContext logContext = new LogContext("testCache ");
 
     @Test
     public void basicPutGet() throws IOException {
@@ -47,22 +52,21 @@ public class ThreadCacheTest {
                 new KeyValue<>("K4", "V4"),
                 new KeyValue<>("K5", "V5"));
         final KeyValue<String, String> kv = toInsert.get(0);
-        final String name = "name";
-        ThreadCache cache = new ThreadCache("testCache",
+        ThreadCache cache = new ThreadCache(logContext,
                 toInsert.size() * memoryCacheEntrySize(kv.key.getBytes(), kv.value.getBytes(), ""),
             new MockStreamsMetrics(new Metrics()));
 
-        for (int i = 0; i < toInsert.size(); i++) {
-            byte[] key = toInsert.get(i).key.getBytes();
-            byte[] value = toInsert.get(i).value.getBytes();
-            cache.put(name, key, new LRUCacheEntry(value, true, 1L, 1L, 1, ""));
+        for (KeyValue<String, String> kvToInsert : toInsert) {
+            Bytes key = Bytes.wrap(kvToInsert.key.getBytes());
+            byte[] value = kvToInsert.value.getBytes();
+            cache.put(namespace, key, new LRUCacheEntry(value, true, 1L, 1L, 1, ""));
         }
 
-        for (int i = 0; i < toInsert.size(); i++) {
-            byte[] key = toInsert.get(i).key.getBytes();
-            LRUCacheEntry entry = cache.get(name, key);
-            assertEquals(entry.isDirty, true);
-            assertEquals(new String(entry.value), toInsert.get(i).value);
+        for (KeyValue<String, String> kvToInsert : toInsert) {
+            Bytes key = Bytes.wrap(kvToInsert.key.getBytes());
+            LRUCacheEntry entry = cache.get(namespace, key);
+            assertEquals(entry.isDirty(), true);
+            assertEquals(new String(entry.value), kvToInsert.value);
         }
         assertEquals(cache.gets(), 5);
         assertEquals(cache.puts(), 5);
@@ -73,22 +77,19 @@ public class ThreadCacheTest {
     private void checkOverheads(double entryFactor, double systemFactor, long desiredCacheSize, int keySizeBytes,
                             int valueSizeBytes) {
         Runtime runtime = Runtime.getRuntime();
-        byte[] key = new byte[keySizeBytes];
-        byte[] value = new byte[valueSizeBytes];
-        final String name = "name";
-        long numElements = desiredCacheSize / memoryCacheEntrySize(key, value, "");
+        long numElements = desiredCacheSize / memoryCacheEntrySize(new byte[keySizeBytes], new byte[valueSizeBytes], "");
 
         System.gc();
         long prevRuntimeMemory = runtime.totalMemory() - runtime.freeMemory();
 
-        ThreadCache cache = new ThreadCache("testCache", desiredCacheSize, new MockStreamsMetrics(new Metrics()));
+        ThreadCache cache = new ThreadCache(logContext, desiredCacheSize, new MockStreamsMetrics(new Metrics()));
         long size = cache.sizeBytes();
         assertEquals(size, 0);
         for (int i = 0; i < numElements; i++) {
             String keyStr = "K" + i;
-            key = keyStr.getBytes();
-            value = new byte[valueSizeBytes];
-            cache.put(name, key, new LRUCacheEntry(value, true, 1L, 1L, 1, ""));
+            Bytes key = Bytes.wrap(keyStr.getBytes());
+            byte[] value = new byte[valueSizeBytes];
+            cache.put(namespace, key, new LRUCacheEntry(value, true, 1L, 1L, 1, ""));
         }
 
 
@@ -144,7 +145,7 @@ public class ThreadCacheTest {
     @Test
     public void evict() throws IOException {
         final List<KeyValue<String, String>> received = new ArrayList<>();
-        List<KeyValue<String, String>> expected = Arrays.asList(
+        List<KeyValue<String, String>> expected = Collections.singletonList(
                 new KeyValue<>("K1", "V1"));
 
         List<KeyValue<String, String>> toInsert = Arrays.asList(
@@ -154,8 +155,7 @@ public class ThreadCacheTest {
                 new KeyValue<>("K4", "V4"),
                 new KeyValue<>("K5", "V5"));
         final KeyValue<String, String> kv = toInsert.get(0);
-        final String namespace = "kafka";
-        ThreadCache cache = new ThreadCache("testCache",
+        ThreadCache cache = new ThreadCache(logContext,
                 memoryCacheEntrySize(kv.key.getBytes(), kv.value.getBytes(), ""),
             new MockStreamsMetrics(new Metrics()));
         cache.addDirtyEntryFlushListener(namespace, new ThreadCache.DirtyEntryFlushListener() {
@@ -168,10 +168,9 @@ public class ThreadCacheTest {
 
         });
 
-
-        for (int i = 0; i < toInsert.size(); i++) {
-            byte[] key = toInsert.get(i).key.getBytes();
-            byte[] value = toInsert.get(i).value.getBytes();
+        for (KeyValue<String, String> kvToInsert : toInsert) {
+            final Bytes key = Bytes.wrap(kvToInsert.key.getBytes());
+            final byte[] value = kvToInsert.value.getBytes();
             cache.put(namespace, key, new LRUCacheEntry(value, true, 1, 1, 1, ""));
         }
 
@@ -184,29 +183,28 @@ public class ThreadCacheTest {
     }
 
     @Test
-    public void shouldDelete() throws Exception {
-        final ThreadCache cache = new ThreadCache("testCache", 10000L, new MockStreamsMetrics(new Metrics()));
-        final byte[] key = new byte[]{0};
+    public void shouldDelete() {
+        final ThreadCache cache = new ThreadCache(logContext, 10000L, new MockStreamsMetrics(new Metrics()));
+        final Bytes key = Bytes.wrap(new byte[]{0});
 
-        cache.put("name", key, dirtyEntry(key));
-        assertEquals(key, cache.delete("name", key).value);
-        assertNull(cache.get("name", key));
+        cache.put(namespace, key, dirtyEntry(key.get()));
+        assertEquals(key.get(), cache.delete(namespace, key).value);
+        assertNull(cache.get(namespace, key));
     }
 
     @Test
-    public void shouldNotFlushAfterDelete() throws Exception {
-        final byte[] key = new byte[]{0};
-        final ThreadCache cache = new ThreadCache("testCache", 10000L, new MockStreamsMetrics(new Metrics()));
+    public void shouldNotFlushAfterDelete() {
+        final Bytes key = Bytes.wrap(new byte[]{0});
+        final ThreadCache cache = new ThreadCache(logContext, 10000L, new MockStreamsMetrics(new Metrics()));
         final List<ThreadCache.DirtyEntry> received = new ArrayList<>();
-        final String namespace = "namespace";
         cache.addDirtyEntryFlushListener(namespace, new ThreadCache.DirtyEntryFlushListener() {
             @Override
             public void apply(final List<ThreadCache.DirtyEntry> dirty) {
                 received.addAll(dirty);
             }
         });
-        cache.put(namespace, key, dirtyEntry(key));
-        assertEquals(key, cache.delete(namespace, key).value);
+        cache.put(namespace, key, dirtyEntry(key.get()));
+        assertEquals(key.get(), cache.delete(namespace, key).value);
 
         // flushing should have no further effect
         cache.flush(namespace);
@@ -215,76 +213,73 @@ public class ThreadCacheTest {
     }
 
     @Test
-    public void shouldNotBlowUpOnNonExistentKeyWhenDeleting() throws Exception {
-        final ThreadCache cache = new ThreadCache("testCache", 10000L, new MockStreamsMetrics(new Metrics()));
-        final byte[] key = new byte[]{0};
+    public void shouldNotBlowUpOnNonExistentKeyWhenDeleting() {
+        final Bytes key = Bytes.wrap(new byte[]{0});
+        final ThreadCache cache = new ThreadCache(logContext, 10000L, new MockStreamsMetrics(new Metrics()));
 
-        cache.put("name", key, dirtyEntry(key));
-        assertNull(cache.delete("name", new byte[]{1}));
+        cache.put(namespace, key, dirtyEntry(key.get()));
+        assertNull(cache.delete(namespace, Bytes.wrap(new byte[]{1})));
     }
 
     @Test
-    public void shouldNotBlowUpOnNonExistentNamespaceWhenDeleting() throws Exception {
-        final ThreadCache cache = new ThreadCache("testCache", 10000L, new MockStreamsMetrics(new Metrics()));
-        assertNull(cache.delete("name", new byte[]{1}));
+    public void shouldNotBlowUpOnNonExistentNamespaceWhenDeleting() {
+        final ThreadCache cache = new ThreadCache(logContext, 10000L, new MockStreamsMetrics(new Metrics()));
+        assertNull(cache.delete(namespace, Bytes.wrap(new byte[]{1})));
     }
 
     @Test
-    public void shouldNotClashWithOverlappingNames() throws Exception {
-        final ThreadCache cache = new ThreadCache("testCache", 10000L, new MockStreamsMetrics(new Metrics()));
-        final byte[] nameByte = new byte[]{0};
-        final byte[] name1Byte = new byte[]{1};
-        cache.put("name", nameByte, dirtyEntry(nameByte));
-        cache.put("name1", nameByte, dirtyEntry(name1Byte));
+    public void shouldNotClashWithOverlappingNames() {
+        final ThreadCache cache = new ThreadCache(logContext, 10000L, new MockStreamsMetrics(new Metrics()));
+        final Bytes nameByte = Bytes.wrap(new byte[]{0});
+        final Bytes name1Byte = Bytes.wrap(new byte[]{1});
+        cache.put(namespace1, nameByte, dirtyEntry(nameByte.get()));
+        cache.put(namespace2, nameByte, dirtyEntry(name1Byte.get()));
 
-        assertArrayEquals(nameByte, cache.get("name", nameByte).value);
-        assertArrayEquals(name1Byte, cache.get("name1", nameByte).value);
+        assertArrayEquals(nameByte.get(), cache.get(namespace1, nameByte).value);
+        assertArrayEquals(name1Byte.get(), cache.get(namespace2, nameByte).value);
     }
 
     @Test
-    public void shouldPeekNextKey() throws Exception {
-        final ThreadCache cache = new ThreadCache("testCache", 10000L, new MockStreamsMetrics(new Metrics()));
-        final byte[] theByte = {0};
-        final String namespace = "streams";
-        cache.put(namespace, theByte, dirtyEntry(theByte));
-        final ThreadCache.MemoryLRUCacheBytesIterator iterator = cache.range(namespace, theByte, new byte[]{1});
-        assertEquals(Bytes.wrap(theByte), iterator.peekNextKey());
-        assertEquals(Bytes.wrap(theByte), iterator.peekNextKey());
+    public void shouldPeekNextKey() {
+        final ThreadCache cache = new ThreadCache(logContext, 10000L, new MockStreamsMetrics(new Metrics()));
+        final Bytes theByte = Bytes.wrap(new byte[]{0});
+        cache.put(namespace, theByte, dirtyEntry(theByte.get()));
+        final ThreadCache.MemoryLRUCacheBytesIterator iterator = cache.range(namespace, theByte, Bytes.wrap(new byte[]{1}));
+        assertEquals(theByte, iterator.peekNextKey());
+        assertEquals(theByte, iterator.peekNextKey());
     }
 
     @Test
-    public void shouldGetSameKeyAsPeekNext() throws Exception {
-        final ThreadCache cache = new ThreadCache("testCache", 10000L, new MockStreamsMetrics(new Metrics()));
-        final byte[] theByte = {0};
-        final String namespace = "streams";
-        cache.put(namespace, theByte, dirtyEntry(theByte));
-        final ThreadCache.MemoryLRUCacheBytesIterator iterator = cache.range(namespace, theByte, new byte[]{1});
+    public void shouldGetSameKeyAsPeekNext() {
+        final ThreadCache cache = new ThreadCache(logContext, 10000L, new MockStreamsMetrics(new Metrics()));
+        final Bytes theByte = Bytes.wrap(new byte[]{0});
+        cache.put(namespace, theByte, dirtyEntry(theByte.get()));
+        final ThreadCache.MemoryLRUCacheBytesIterator iterator = cache.range(namespace, theByte, Bytes.wrap(new byte[]{1}));
         assertEquals(iterator.peekNextKey(), iterator.next().key);
     }
 
     @Test(expected = NoSuchElementException.class)
-    public void shouldThrowIfNoPeekNextKey() throws Exception {
-        final ThreadCache cache = new ThreadCache("testCache", 10000L, new MockStreamsMetrics(new Metrics()));
-        final ThreadCache.MemoryLRUCacheBytesIterator iterator = cache.range("", new byte[]{0}, new byte[]{1});
+    public void shouldThrowIfNoPeekNextKey() {
+        final ThreadCache cache = new ThreadCache(logContext, 10000L, new MockStreamsMetrics(new Metrics()));
+        final ThreadCache.MemoryLRUCacheBytesIterator iterator = cache.range(namespace, Bytes.wrap(new byte[]{0}), Bytes.wrap(new byte[]{1}));
         iterator.peekNextKey();
     }
 
     @Test
-    public void shouldReturnFalseIfNoNextKey() throws Exception {
-        final ThreadCache cache = new ThreadCache("testCache", 10000L, new MockStreamsMetrics(new Metrics()));
-        final ThreadCache.MemoryLRUCacheBytesIterator iterator = cache.range("", new byte[]{0}, new byte[]{1});
+    public void shouldReturnFalseIfNoNextKey() {
+        final ThreadCache cache = new ThreadCache(logContext, 10000L, new MockStreamsMetrics(new Metrics()));
+        final ThreadCache.MemoryLRUCacheBytesIterator iterator = cache.range(namespace, Bytes.wrap(new byte[]{0}), Bytes.wrap(new byte[]{1}));
         assertFalse(iterator.hasNext());
     }
 
     @Test
-    public void shouldPeekAndIterateOverRange() throws Exception {
-        final ThreadCache cache = new ThreadCache("testCache", 10000L, new MockStreamsMetrics(new Metrics()));
+    public void shouldPeekAndIterateOverRange() {
+        final ThreadCache cache = new ThreadCache(logContext, 10000L, new MockStreamsMetrics(new Metrics()));
         final byte[][] bytes = {{0}, {1}, {2}, {3}, {4}, {5}, {6}, {7}, {8}, {9}, {10}};
-        final String namespace = "streams";
         for (final byte[] aByte : bytes) {
-            cache.put(namespace, aByte, dirtyEntry(aByte));
+            cache.put(namespace, Bytes.wrap(aByte), dirtyEntry(aByte));
         }
-        final ThreadCache.MemoryLRUCacheBytesIterator iterator = cache.range(namespace, new byte[]{1}, new byte[]{4});
+        final ThreadCache.MemoryLRUCacheBytesIterator iterator = cache.range(namespace, Bytes.wrap(new byte[]{1}), Bytes.wrap(new byte[]{4}));
         int bytesIndex = 1;
         while (iterator.hasNext()) {
             Bytes peekedKey = iterator.peekNextKey();
@@ -297,10 +292,9 @@ public class ThreadCacheTest {
     }
 
     @Test
-    public void shouldSkipEntriesWhereValueHasBeenEvictedFromCache() throws Exception {
-        final String namespace = "streams";
+    public void shouldSkipEntriesWhereValueHasBeenEvictedFromCache() {
         final int entrySize = memoryCacheEntrySize(new byte[1], new byte[1], "");
-        final ThreadCache cache = new ThreadCache("testCache", entrySize * 5, new MockStreamsMetrics(new Metrics()));
+        final ThreadCache cache = new ThreadCache(logContext, entrySize * 5, new MockStreamsMetrics(new Metrics()));
         cache.addDirtyEntryFlushListener(namespace, new ThreadCache.DirtyEntryFlushListener() {
             @Override
             public void apply(final List<ThreadCache.DirtyEntry> dirty) {
@@ -309,22 +303,22 @@ public class ThreadCacheTest {
         });
         byte[][] bytes = {{0}, {1}, {2}, {3}, {4}, {5}, {6}, {7}, {8}, {9}};
         for (int i = 0; i < 5; i++) {
-            cache.put(namespace, bytes[i], dirtyEntry(bytes[i]));
+            cache.put(namespace, Bytes.wrap(bytes[i]), dirtyEntry(bytes[i]));
         }
         assertEquals(5, cache.size());
 
-        final ThreadCache.MemoryLRUCacheBytesIterator range = cache.range(namespace, new byte[]{0}, new byte[]{5});
+        final ThreadCache.MemoryLRUCacheBytesIterator range = cache.range(namespace, Bytes.wrap(new byte[]{0}), Bytes.wrap(new byte[]{5}));
         // should evict byte[] {0}
-        cache.put(namespace, new byte[]{6}, dirtyEntry(new byte[]{6}));
+        cache.put(namespace, Bytes.wrap(new byte[]{6}), dirtyEntry(new byte[]{6}));
 
         assertEquals(Bytes.wrap(new byte[]{1}), range.peekNextKey());
     }
 
     @Test
-    public void shouldFlushDirtyEntriesForNamespace() throws Exception {
-        final ThreadCache cache = new ThreadCache("testCache", 100000, new MockStreamsMetrics(new Metrics()));
+    public void shouldFlushDirtyEntriesForNamespace() {
+        final ThreadCache cache = new ThreadCache(logContext, 100000, new MockStreamsMetrics(new Metrics()));
         final List<byte[]> received = new ArrayList<>();
-        cache.addDirtyEntryFlushListener("1", new ThreadCache.DirtyEntryFlushListener() {
+        cache.addDirtyEntryFlushListener(namespace1, new ThreadCache.DirtyEntryFlushListener() {
             @Override
             public void apply(final List<ThreadCache.DirtyEntry> dirty) {
                 for (ThreadCache.DirtyEntry dirtyEntry : dirty) {
@@ -334,19 +328,19 @@ public class ThreadCacheTest {
         });
         final List<byte[]> expected = Arrays.asList(new byte[]{0}, new byte[]{1}, new byte[]{2});
         for (byte[] bytes : expected) {
-            cache.put("1", bytes, dirtyEntry(bytes));
+            cache.put(namespace1, Bytes.wrap(bytes), dirtyEntry(bytes));
         }
-        cache.put("2", new byte[]{4}, dirtyEntry(new byte[]{4}));
+        cache.put(namespace2, Bytes.wrap(new byte[]{4}), dirtyEntry(new byte[]{4}));
 
-        cache.flush("1");
+        cache.flush(namespace1);
         assertEquals(expected, received);
     }
 
     @Test
-    public void shouldNotFlushCleanEntriesForNamespace() throws Exception {
-        final ThreadCache cache = new ThreadCache("testCache", 100000, new MockStreamsMetrics(new Metrics()));
+    public void shouldNotFlushCleanEntriesForNamespace() {
+        final ThreadCache cache = new ThreadCache(logContext, 100000, new MockStreamsMetrics(new Metrics()));
         final List<byte[]> received = new ArrayList<>();
-        cache.addDirtyEntryFlushListener("1", new ThreadCache.DirtyEntryFlushListener() {
+        cache.addDirtyEntryFlushListener(namespace1, new ThreadCache.DirtyEntryFlushListener() {
             @Override
             public void apply(final List<ThreadCache.DirtyEntry> dirty) {
                 for (ThreadCache.DirtyEntry dirtyEntry : dirty) {
@@ -356,25 +350,25 @@ public class ThreadCacheTest {
         });
         final List<byte[]> toInsert =  Arrays.asList(new byte[]{0}, new byte[]{1}, new byte[]{2});
         for (byte[] bytes : toInsert) {
-            cache.put("1", bytes, cleanEntry(bytes));
+            cache.put(namespace1, Bytes.wrap(bytes), cleanEntry(bytes));
         }
-        cache.put("2", new byte[]{4}, cleanEntry(new byte[]{4}));
+        cache.put(namespace2, Bytes.wrap(new byte[]{4}), cleanEntry(new byte[]{4}));
 
-        cache.flush("1");
+        cache.flush(namespace1);
         assertEquals(Collections.EMPTY_LIST, received);
     }
 
 
     private void shouldEvictImmediatelyIfCacheSizeIsZeroOrVerySmall(final ThreadCache cache) {
         final List<ThreadCache.DirtyEntry> received = new ArrayList<>();
-        final String namespace = "namespace";
+
         cache.addDirtyEntryFlushListener(namespace, new ThreadCache.DirtyEntryFlushListener() {
             @Override
             public void apply(final List<ThreadCache.DirtyEntry> dirty) {
                 received.addAll(dirty);
             }
         });
-        cache.put(namespace, new byte[]{0}, dirtyEntry(new byte[]{0}));
+        cache.put(namespace, Bytes.wrap(new byte[]{0}), dirtyEntry(new byte[]{0}));
         assertEquals(1, received.size());
 
         // flushing should have no further effect
@@ -383,22 +377,21 @@ public class ThreadCacheTest {
     }
 
     @Test
-    public void shouldEvictImmediatelyIfCacheSizeIsVerySmall() throws Exception {
-        final ThreadCache cache = new ThreadCache("testCache", 1, new MockStreamsMetrics(new Metrics()));
+    public void shouldEvictImmediatelyIfCacheSizeIsVerySmall() {
+        final ThreadCache cache = new ThreadCache(logContext, 1, new MockStreamsMetrics(new Metrics()));
         shouldEvictImmediatelyIfCacheSizeIsZeroOrVerySmall(cache);
     }
 
     @Test
-    public void shouldEvictImmediatelyIfCacheSizeIsZero() throws Exception {
-        final ThreadCache cache = new ThreadCache("testCache", 0, new MockStreamsMetrics(new Metrics()));
+    public void shouldEvictImmediatelyIfCacheSizeIsZero() {
+        final ThreadCache cache = new ThreadCache(logContext, 0, new MockStreamsMetrics(new Metrics()));
         shouldEvictImmediatelyIfCacheSizeIsZeroOrVerySmall(cache);
     }
 
     @Test
-    public void shouldEvictAfterPutAll() throws Exception {
+    public void shouldEvictAfterPutAll() {
         final List<ThreadCache.DirtyEntry> received = new ArrayList<>();
-        final String namespace = "namespace";
-        final ThreadCache cache = new ThreadCache("testCache", 1, new MockStreamsMetrics(new Metrics()));
+        final ThreadCache cache = new ThreadCache(logContext, 1, new MockStreamsMetrics(new Metrics()));
         cache.addDirtyEntryFlushListener(namespace, new ThreadCache.DirtyEntryFlushListener() {
             @Override
             public void apply(final List<ThreadCache.DirtyEntry> dirty) {
@@ -406,51 +399,51 @@ public class ThreadCacheTest {
             }
         });
 
-        cache.putAll(namespace, Arrays.asList(KeyValue.pair(new byte[]{0}, dirtyEntry(new byte[]{5})),
-            KeyValue.pair(new byte[]{1}, dirtyEntry(new byte[]{6}))));
+        cache.putAll(namespace, Arrays.asList(KeyValue.pair(Bytes.wrap(new byte[]{0}), dirtyEntry(new byte[]{5})),
+                                              KeyValue.pair(Bytes.wrap(new byte[]{1}), dirtyEntry(new byte[]{6}))));
 
         assertEquals(cache.evicts(), 2);
+        assertEquals(received.size(), 2);
     }
 
     @Test
-    public void shouldPutAll() throws Exception {
-        final ThreadCache cache = new ThreadCache("testCache", 100000, new MockStreamsMetrics(new Metrics()));
+    public void shouldPutAll() {
+        final ThreadCache cache = new ThreadCache(logContext, 100000, new MockStreamsMetrics(new Metrics()));
 
-        cache.putAll("name", Arrays.asList(KeyValue.pair(new byte[]{0}, dirtyEntry(new byte[]{5})),
-                                           KeyValue.pair(new byte[]{1}, dirtyEntry(new byte[]{6}))));
+        cache.putAll(namespace, Arrays.asList(KeyValue.pair(Bytes.wrap(new byte[]{0}), dirtyEntry(new byte[]{5})),
+                                           KeyValue.pair(Bytes.wrap(new byte[]{1}), dirtyEntry(new byte[]{6}))));
 
-        assertArrayEquals(new byte[]{5}, cache.get("name", new byte[]{0}).value);
-        assertArrayEquals(new byte[]{6}, cache.get("name", new byte[]{1}).value);
+        assertArrayEquals(new byte[]{5}, cache.get(namespace, Bytes.wrap(new byte[]{0})).value);
+        assertArrayEquals(new byte[]{6}, cache.get(namespace, Bytes.wrap(new byte[]{1})).value);
     }
 
     @Test
-    public void shouldNotForwardCleanEntryOnEviction() throws Exception {
-        final ThreadCache cache = new ThreadCache("testCache", 0, new MockStreamsMetrics(new Metrics()));
+    public void shouldNotForwardCleanEntryOnEviction() {
+        final ThreadCache cache = new ThreadCache(logContext, 0, new MockStreamsMetrics(new Metrics()));
         final List<ThreadCache.DirtyEntry> received = new ArrayList<>();
-        cache.addDirtyEntryFlushListener("name", new ThreadCache.DirtyEntryFlushListener() {
+        cache.addDirtyEntryFlushListener(namespace, new ThreadCache.DirtyEntryFlushListener() {
             @Override
             public void apply(final List<ThreadCache.DirtyEntry> dirty) {
                 received.addAll(dirty);
             }
         });
-        cache.put("name", new byte[] {1}, cleanEntry(new byte[]{0}));
+        cache.put(namespace, Bytes.wrap(new byte[]{1}), cleanEntry(new byte[]{0}));
         assertEquals(0, received.size());
     }
     @Test
-    public void shouldPutIfAbsent() throws Exception {
-        final ThreadCache cache = new ThreadCache("testCache", 100000, new MockStreamsMetrics(new Metrics()));
-        final byte[] key = {10};
+    public void shouldPutIfAbsent() {
+        final ThreadCache cache = new ThreadCache(logContext, 100000, new MockStreamsMetrics(new Metrics()));
+        final Bytes key = Bytes.wrap(new byte[]{10});
         final byte[] value = {30};
-        assertNull(cache.putIfAbsent("n", key, dirtyEntry(value)));
-        assertArrayEquals(value, cache.putIfAbsent("n", key, dirtyEntry(new byte[]{8})).value);
-        assertArrayEquals(value, cache.get("n", key).value);
+        assertNull(cache.putIfAbsent(namespace, key, dirtyEntry(value)));
+        assertArrayEquals(value, cache.putIfAbsent(namespace, key, dirtyEntry(new byte[]{8})).value);
+        assertArrayEquals(value, cache.get(namespace, key).value);
     }
 
     @Test
-    public void shouldEvictAfterPutIfAbsent() throws Exception {
+    public void shouldEvictAfterPutIfAbsent() {
         final List<ThreadCache.DirtyEntry> received = new ArrayList<>();
-        final String namespace = "namespace";
-        final ThreadCache cache = new ThreadCache("testCache", 1, new MockStreamsMetrics(new Metrics()));
+        final ThreadCache cache = new ThreadCache(logContext, 1, new MockStreamsMetrics(new Metrics()));
         cache.addDirtyEntryFlushListener(namespace, new ThreadCache.DirtyEntryFlushListener() {
             @Override
             public void apply(final List<ThreadCache.DirtyEntry> dirty) {
@@ -458,63 +451,72 @@ public class ThreadCacheTest {
             }
         });
 
-        cache.putIfAbsent(namespace, new byte[]{0}, dirtyEntry(new byte[]{5}));
-        cache.putIfAbsent(namespace, new byte[]{1}, dirtyEntry(new byte[]{6}));
-        cache.putIfAbsent(namespace, new byte[]{1}, dirtyEntry(new byte[]{6}));
+        cache.putIfAbsent(namespace, Bytes.wrap(new byte[]{0}), dirtyEntry(new byte[]{5}));
+        cache.putIfAbsent(namespace, Bytes.wrap(new byte[]{1}), dirtyEntry(new byte[]{6}));
+        cache.putIfAbsent(namespace, Bytes.wrap(new byte[]{1}), dirtyEntry(new byte[]{6}));
 
         assertEquals(cache.evicts(), 3);
+        assertEquals(received.size(), 3);
     }
 
     @Test
-    public void shouldNotLoopForEverWhenEvictingAndCurrentCacheIsEmpty() throws Exception {
+    public void shouldNotLoopForEverWhenEvictingAndCurrentCacheIsEmpty() {
         final int maxCacheSizeInBytes = 100;
-        final ThreadCache threadCache = new ThreadCache("testCache", maxCacheSizeInBytes, new MockStreamsMetrics(new Metrics()));
+        final ThreadCache threadCache = new ThreadCache(logContext, maxCacheSizeInBytes, new MockStreamsMetrics(new Metrics()));
         // trigger a put into another cache on eviction from "name"
-        threadCache.addDirtyEntryFlushListener("name", new ThreadCache.DirtyEntryFlushListener() {
+        threadCache.addDirtyEntryFlushListener(namespace, new ThreadCache.DirtyEntryFlushListener() {
             @Override
             public void apply(final List<ThreadCache.DirtyEntry> dirty) {
                 // put an item into an empty cache when the total cache size
                 // is already > than maxCacheSizeBytes
-                threadCache.put("other", new byte[]{0}, dirtyEntry(new byte[2]));
+                threadCache.put(namespace1, Bytes.wrap(new byte[]{0}), dirtyEntry(new byte[2]));
             }
         });
-        threadCache.addDirtyEntryFlushListener("other", new ThreadCache.DirtyEntryFlushListener() {
+        threadCache.addDirtyEntryFlushListener(namespace1, new ThreadCache.DirtyEntryFlushListener() {
             @Override
             public void apply(final List<ThreadCache.DirtyEntry> dirty) {
                //
             }
         });
-        threadCache.addDirtyEntryFlushListener("another", new ThreadCache.DirtyEntryFlushListener() {
+        threadCache.addDirtyEntryFlushListener(namespace2, new ThreadCache.DirtyEntryFlushListener() {
             @Override
             public void apply(final List<ThreadCache.DirtyEntry> dirty) {
 
             }
         });
 
-        threadCache.put("another", new byte[]{1}, dirtyEntry(new byte[1]));
-        threadCache.put("name", new byte[]{1}, dirtyEntry(new byte[1]));
+        threadCache.put(namespace2, Bytes.wrap(new byte[]{1}), dirtyEntry(new byte[1]));
+        threadCache.put(namespace, Bytes.wrap(new byte[]{1}), dirtyEntry(new byte[1]));
         // Put a large item such that when the eldest item is removed
         // cache sizeInBytes() > maxCacheSizeBytes
         int remaining = (int) (maxCacheSizeInBytes - threadCache.sizeBytes());
-        threadCache.put("name", new byte[]{2}, dirtyEntry(new byte[remaining + 100]));
+        threadCache.put(namespace, Bytes.wrap(new byte[]{2}), dirtyEntry(new byte[remaining + 100]));
     }
 
     @Test
-    public void shouldCleanupNamedCacheOnClose() throws Exception {
-        final ThreadCache cache = new ThreadCache("testCache", 100000, new MockStreamsMetrics(new Metrics()));
-        cache.put("one", new byte[]{1}, cleanEntry(new byte[] {1}));
-        cache.put("two", new byte[]{1}, cleanEntry(new byte[] {1}));
+    public void shouldCleanupNamedCacheOnClose() {
+        final ThreadCache cache = new ThreadCache(logContext, 100000, new MockStreamsMetrics(new Metrics()));
+        cache.put(namespace1, Bytes.wrap(new byte[]{1}), cleanEntry(new byte[] {1}));
+        cache.put(namespace2, Bytes.wrap(new byte[]{1}), cleanEntry(new byte[] {1}));
         assertEquals(cache.size(), 2);
-        cache.close("two");
+        cache.close(namespace2);
         assertEquals(cache.size(), 1);
-        assertNull(cache.get("two", new byte[] {1}));
+        assertNull(cache.get(namespace2, Bytes.wrap(new byte[]{1})));
     }
 
     @Test
-    public void shouldReturnNullIfKeyIsNull() throws Exception {
-        final ThreadCache threadCache = new ThreadCache("testCache", 10, new MockStreamsMetrics(new Metrics()));
-        threadCache.put("one", new byte[]{1}, cleanEntry(new byte[] {1}));
-        assertNull(threadCache.get("one", null));
+    public void shouldReturnNullIfKeyIsNull() {
+        final ThreadCache threadCache = new ThreadCache(logContext, 10, new MockStreamsMetrics(new Metrics()));
+        threadCache.put(namespace, Bytes.wrap(new byte[]{1}), cleanEntry(new byte[] {1}));
+        assertNull(threadCache.get(namespace, null));
+    }
+
+    @Test
+    public void shouldCalculateSizeInBytes() {
+        final ThreadCache cache = new ThreadCache(logContext, 100000, new MockStreamsMetrics(new Metrics()));
+        NamedCache.LRUNode node = new NamedCache.LRUNode(Bytes.wrap(new byte[]{1}), dirtyEntry(new byte[]{0}));
+        cache.put(namespace1, Bytes.wrap(new byte[]{1}), cleanEntry(new byte[]{0}));
+        assertEquals(cache.sizeBytes(), node.size());
     }
 
     private LRUCacheEntry dirtyEntry(final byte[] key) {
