@@ -1,10 +1,10 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
+ * contributor license agreements. See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
+ * the License. You may obtain a copy of the License at
  *
  *    http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -14,10 +14,11 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.kafka.test;
 
 import org.apache.kafka.streams.processor.ProcessorContext;
+import org.apache.kafka.streams.processor.PunctuationType;
+import org.apache.kafka.streams.processor.Punctuator;
 import org.apache.kafka.streams.processor.internals.ProcessorNode;
 
 import java.util.Collections;
@@ -25,16 +26,20 @@ import java.util.concurrent.atomic.AtomicInteger;
 
 public class MockProcessorNode<K, V> extends ProcessorNode<K, V> {
 
-    public static final String NAME = "MOCK-PROCESS-";
-    public static final AtomicInteger INDEX = new AtomicInteger(1);
-
-    public int numReceived = 0;
+    private static final String NAME = "MOCK-PROCESS-";
+    private static final AtomicInteger INDEX = new AtomicInteger(1);
 
     public final MockProcessorSupplier<K, V> supplier;
+    public boolean closed;
+    public long punctuatedAt;
     public boolean initialized;
 
     public MockProcessorNode(long scheduleInterval) {
-        this(new MockProcessorSupplier<K, V>(scheduleInterval));
+        this(scheduleInterval, PunctuationType.STREAM_TIME);
+    }
+
+    public MockProcessorNode(long scheduleInterval, PunctuationType punctuationType) {
+        this(new MockProcessorSupplier<K, V>(scheduleInterval, punctuationType));
     }
 
     private MockProcessorNode(MockProcessorSupplier<K, V> supplier) {
@@ -51,7 +56,18 @@ public class MockProcessorNode<K, V> extends ProcessorNode<K, V> {
 
     @Override
     public void process(K key, V value) {
-        this.numReceived++;
         processor().process(key, value);
     }
+
+    @Override
+    public void punctuate(final long timestamp, final Punctuator punctuator) {
+        super.punctuate(timestamp, punctuator);
+        this.punctuatedAt = timestamp;
+    }
+
+    @Override
+    public void close() {
+        super.close();
+        this.closed = true;
+    }
 }
\ No newline at end of file
