@@ -24,7 +24,7 @@ class KafkaConfig(dict):
     DEFAULTS = {
         config_property.PORT: 9092,
         config_property.SOCKET_RECEIVE_BUFFER_BYTES: 65536,
-        config_property.LOG_DIRS: "/mnt/kafka-data-logs",
+        config_property.LOG_DIRS: "/mnt/kafka/kafka-data-logs-1,/mnt/kafka/kafka-data-logs-2",
         config_property.ZOOKEEPER_CONNECTION_TIMEOUT_MS: 2000
     }
 
