@@ -14,6 +14,7 @@
 # limitations under the License.
 
 
+import os
 import re
 import time
 
@@ -27,13 +28,15 @@ from kafkatest.version import DEV_BRANCH
 
 
 class ZookeeperService(KafkaPathResolverMixin, Service):
+    ROOT = "/mnt/zookeeper"
+    DATA = os.path.join(ROOT, "data")
 
     logs = {
         "zk_log": {
-            "path": "/mnt/zk.log",
+            "path": "%s/zk.log" % ROOT,
             "collect_default": True},
         "zk_data": {
-            "path": "/mnt/zookeeper",
+            "path": DATA,
             "collect_default": False}
     }
 
@@ -64,19 +67,19 @@ class ZookeeperService(KafkaPathResolverMixin, Service):
         idx = self.idx(node)
         self.logger.info("Starting ZK node %d on %s", idx, node.account.hostname)
 
-        node.account.ssh("mkdir -p /mnt/zookeeper")
-        node.account.ssh("echo %d > /mnt/zookeeper/myid" % idx)
+        node.account.ssh("mkdir -p %s" % ZookeeperService.DATA)
+        node.account.ssh("echo %d > %s/myid" % (idx, ZookeeperService.DATA))
 
         self.security_config.setup_node(node)
         config_file = self.render('zookeeper.properties')
         self.logger.info("zookeeper.properties:")
         self.logger.info(config_file)
-        node.account.create_file("/mnt/zookeeper.properties", config_file)
+        node.account.create_file("%s/zookeeper.properties" % ZookeeperService.ROOT, config_file)
 
         start_cmd = "export KAFKA_OPTS=\"%s\";" % (self.kafka_opts + ' ' + self.security_system_properties) \
             if self.security_config.zk_sasl else self.kafka_opts
         start_cmd += "%s " % self.path.script("zookeeper-server-start.sh", node)
-        start_cmd += "/mnt/zookeeper.properties 1>> %(path)s 2>> %(path)s &" % self.logs["zk_log"]
+        start_cmd += "%s/zookeeper.properties &>> %s &" % (ZookeeperService.ROOT, self.logs["zk_log"]["path"])
         node.account.ssh(start_cmd)
 
         wait_until(lambda: self.listening(node), timeout_sec=30, err_msg="Zookeeper node failed to start")
@@ -91,12 +94,7 @@ class ZookeeperService(KafkaPathResolverMixin, Service):
             return False
 
     def pids(self, node):
-        try:
-            cmd = "ps ax | grep -i zookeeper | grep java | grep -v grep | awk '{print $1}'"
-            pid_arr = [pid for pid in node.account.ssh_capture(cmd, allow_fail=True, callback=int)]
-            return pid_arr
-        except (RemoteCommandError, ValueError) as e:
-            return []
+        return node.account.java_pids(self.java_class_name())
 
     def alive(self, node):
         return len(self.pids(node)) > 0
@@ -104,7 +102,8 @@ class ZookeeperService(KafkaPathResolverMixin, Service):
     def stop_node(self, node):
         idx = self.idx(node)
         self.logger.info("Stopping %s node %d on %s" % (type(self).__name__, idx, node.account.hostname))
-        node.account.kill_process("zookeeper", allow_fail=False)
+        node.account.kill_java_processes(self.java_class_name(), allow_fail=False)
+        node.account.kill_java_processes(self.java_cli_class_name(), allow_fail=False)
         wait_until(lambda: not self.alive(node), timeout_sec=5, err_msg="Timed out waiting for zookeeper to stop.")
 
     def clean_node(self, node):
@@ -112,11 +111,19 @@ class ZookeeperService(KafkaPathResolverMixin, Service):
         if self.alive(node):
             self.logger.warn("%s %s was still alive at cleanup time. Killing forcefully..." %
                              (self.__class__.__name__, node.account))
-        node.account.kill_process("zookeeper", clean_shutdown=False, allow_fail=True)
-        node.account.ssh("rm -rf /mnt/zookeeper /mnt/zookeeper.properties /mnt/zk.log", allow_fail=False)
+        node.account.kill_java_processes(self.java_class_name(),
+                                         clean_shutdown=False, allow_fail=True)
+        node.account.kill_java_processes(self.java_cli_class_name(),
+                                         clean_shutdown=False, allow_fail=False)
+        node.account.ssh("rm -rf -- %s" % ZookeeperService.ROOT, allow_fail=False)
 
-    def connect_setting(self):
-        return ','.join([node.account.hostname + ':2181' for node in self.nodes])
+
+    def connect_setting(self, chroot=None):
+        if chroot and not chroot.startswith("/"):
+            raise Exception("ZK chroot must start with '/', invalid chroot: %s" % chroot)
+
+        chroot = '' if chroot is None else chroot
+        return ','.join([node.account.hostname + ':2181' + chroot for node in self.nodes])
 
     #
     # This call is used to simulate a rolling upgrade to enable/disable
@@ -127,13 +134,21 @@ class ZookeeperService(KafkaPathResolverMixin, Service):
                        (self.path.script("zookeeper-security-migration.sh", node), zk_acl, self.connect_setting())
         node.account.ssh(la_migra_cmd)
 
-    def query(self, path):
+    def _check_chroot(self, chroot):
+        if chroot and not chroot.startswith("/"):
+            raise Exception("ZK chroot must start with '/', invalid chroot: %s" % chroot)
+
+    def query(self, path, chroot=None):
         """
         Queries zookeeper for data associated with 'path' and returns all fields in the schema
         """
+        self._check_chroot(chroot)
+
+        chroot_path = ('' if chroot is None else chroot) + path
+
         kafka_run_class = self.path.script("kafka-run-class.sh", DEV_BRANCH)
-        cmd = "%s kafka.tools.ZooKeeperMainWrapper -server %s get %s" % \
-              (kafka_run_class, self.connect_setting(), path)
+        cmd = "%s %s -server %s get %s" % \
+              (kafka_run_class, self.java_cli_class_name(), self.connect_setting(), chroot_path)
         self.logger.debug(cmd)
 
         node = self.nodes[0]
@@ -145,3 +160,26 @@ class ZookeeperService(KafkaPathResolverMixin, Service):
                 if match is not None:
                     result = match.groups()[0]
         return result
+
+    def create(self, path, chroot=None):
+        """
+        Create an znode at the given path
+        """
+        self._check_chroot(chroot)
+
+        chroot_path = ('' if chroot is None else chroot) + path
+
+        kafka_run_class = self.path.script("kafka-run-class.sh", DEV_BRANCH)
+        cmd = "%s %s -server %s create %s ''" % \
+              (kafka_run_class, self.java_cli_class_name(), self.connect_setting(), chroot_path)
+        self.logger.debug(cmd)
+        output = self.nodes[0].account.ssh_output(cmd)
+        self.logger.debug(output)
+
+    def java_class_name(self):
+        """ The class name of the Zookeeper quorum peers. """
+        return "org.apache.zookeeper.server.quorum.QuorumPeerMain"
+
+    def java_cli_class_name(self):
+        """ The class name of the Zookeeper tool within Kafka. """
+        return "kafka.tools.ZooKeeperMainWrapper"
